@@ -2,7 +2,7 @@
 #   MailScanner - SMTP E-Mail Virus Scanner
 #   Copyright (C) 2002  Julian Field
 #
-#   $Id: Config.pm,v 1.81.2.54 2004/04/11 17:30:00 jkf Exp $
+#   $Id: Config.pm,v 1.81.2.65 2004/12/18 20:56:36 jkf Exp $
 #
 #   This program is free software; you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
@@ -39,7 +39,7 @@ no  strict 'subs'; # Allow bare words for parameter %'s
 use vars qw($VERSION);
 
 ### The package version, both in 1.23 style *and* usable by MakeMaker:
-$VERSION = substr q$Revision: 1.81.2.54 $, 10;
+$VERSION = substr q$Revision: 1.81.2.65 $, 10;
 
 #1 exception to general rule of forming replies to Config::Value()
 #is %Config::ScannerCmds
@@ -64,7 +64,7 @@ my(%StaticScalars, %ScannerCmds, %SpamLists);
 my(%KeywordCategory, %FilenameRules, %FiletypeRules);
 my(%LanguageStrings);
 my(%RuleScalars, %Defaults, $DefaultAddressRegexp, $DefaultVirusRegexp);
-my(%CustomFunctions);
+my(%CustomFunctions, %CustomFunctionsParams);
 my(%PercentVars); # For putting substituted variables in all settings
 my($RequireLDAPDone);
 use vars qw($LDAP $LDAPserver $LDAPbase $LDAPsite); # LDAP connection info
@@ -78,6 +78,7 @@ $RequireLDAPDone = 0; # Have we done the "require Net::LDAP"?
 %RuleScalars = (); # These are the ones created from rulesets
 
 %CustomFunctions = (); # These are names of user-written functions
+%CustomFunctionsParams = (); # and their parameters passed to Init and End
 
 # This is what the RuleToRegexp function produces when given
 # either "*@*" or "default".
@@ -517,6 +518,15 @@ sub IsSimpleValue {
   return 0;
 }
 
+#
+# Substitute Percent Variables into a line of text
+#
+sub DoPercentVars {
+  my($string) = @_;
+  $string =~ s/\%([^%]+)\%/$PercentVars{lc($1)}/g;
+  $string =~ s/\\n/\n/g;
+  $string;
+}
 
 #
 # Read all the CustomConfig.pm files in the Custom Config Dir
@@ -700,6 +710,11 @@ sub EtoI {
 # Process and setup the configuration
 sub Read {
   my($filename) = @_;
+
+  # Save the configuration filename and directory for later potential use
+  # in Custom Functions.
+  $MailScanner::Config::ConfFile = $filename;
+
   ReadData($filename);
 
   # Read all the filename rules. The "Value" of filenamerules is a list
@@ -744,14 +759,21 @@ sub Read {
   %SpamLists = ReadDefinitions(Value('spamlistdefinitions'));
 
   # Call all the user's custom initialisation functions
-  my($custom, $fn);
-  foreach $custom (values %CustomFunctions) {
+  my($key, $param, $custom, $fn);
+  foreach $key (keys %CustomFunctions) {
+    $custom = $CustomFunctions{$key};
     next unless $custom;
-    MailScanner::Log::InfoLog("Config: calling custom init function %s",
-                              $custom);
-    $fn = 'MailScanner::CustomConfig::Init' . $custom;
+    $param = $CustomFunctionsParams{$key};
+    MailScanner::Log::InfoLog("Config: calling custom init function %s%s",
+                              $custom, $param);
+    $fn = 'MailScanner::CustomConfig::Init' . $custom . $param;
     no strict 'refs';
-    &$fn();
+    eval ($fn);
+    if ($@) {
+      MailScanner::Log::WarnLog("Could not use Custom Function code %s, " .
+        "it could not be \"eval\"ed. Make sure the module is correct with " .
+        "perl -wc", $fn);
+    }
     use strict 'refs';
   }
 
@@ -769,14 +791,16 @@ sub Read {
 # Give all the user's custom functions a chance to clear up
 # and neatly shutdown, log totals, close databases, etc.
 sub EndCustomFunctions {
-  my($custom, $fn);
-  foreach $custom (values %CustomFunctions) {
+  my($custom, $key, $param, $fn);
+  foreach $key (keys %CustomFunctions) {
+    $custom = $CustomFunctions{$key};
     next unless $custom;
-    MailScanner::Log::InfoLog("Config: calling custom end function %s",
-                              $custom);
-    $fn = 'MailScanner::CustomConfig::End' . $custom;
+    $param = $CustomFunctionsParams{$key};
+    MailScanner::Log::InfoLog("Config: calling custom end function %s%s",
+                              $custom, $param);
+    $fn = 'MailScanner::CustomConfig::End' . $custom . $param;
     no strict 'refs';
-    &$fn();
+    eval($fn);
     use strict 'refs';
   }
 }
@@ -1280,8 +1304,8 @@ sub ReadInQueueDirs {
   # Open and lock the list file
   $listh = new FileHandle;
   $listh->open("<$name")
-    or MailScanner::Log::DieLog("File containing list of incoming queue dirs" .
-                                " (%s) does not exist", $name);
+    or MailScanner::Log::WarnLog("File containing list of incoming queue dirs" .
+                                " (%s) does not exist", $name),return;
   flock($listh, $LOCK_SH);
 
   while(<$listh>) {
@@ -1613,7 +1637,7 @@ sub ReadData {
       #print STDERR "Unrecognised keyword \"" . ItoE($leftover) .
       #             "\" at line " . $LineNos{$leftover} . "\n";
     }
-    MailScanner::Log::DieLog("Aborting due to syntax errors in %s.",
+    MailScanner::Log::WarnLog("Aborting due to syntax errors in %s.",
                              $filename);
   }
 }
@@ -1695,7 +1719,7 @@ sub RuleToRegexp {
 
     $file = new FileHandle;
     $file->open("<$rule")
-      or MailScanner::Log::DieLog("Could not open ruleset's address pattern list file %s, %s", $rule, $!);
+      or MailScanner::Log::WarnLog("Could not open ruleset's address pattern list file %s, %s", $rule, $!),return;
     flock($file, $LOCK_SH);
 
     MailScanner::Log::DebugLog("Reading ruleset's address pattern list file %s", $rule);
@@ -1810,6 +1834,7 @@ sub RuleToRegexp {
     # Change . into \., @ into \@, * into .*
     $rule =~ s/\@/\\@/g;
     $rule =~ s/\./\\./g;
+    $rule =~ s/\+/\\+/g;
     $rule =~ s/\*/.*/g;
     # and tack on the optional "." at the end
     $rule .= '\.?';
@@ -1870,7 +1895,7 @@ sub ReadRuleset {
   # If the rulesfilename ends in ".RuleSet" and doesn't contain any '/'
   # characters, then it's an LDAP ruleset.
   #if ($LDAP && $rulesfilename !~ /\// && $rulesfilename =~ /\.RuleSet$/) {
-  if ($LDAP && $rulesfilename =~ /customize/) {
+  if ($LDAP && $rulesfilename =~ /customi[sz]e/) {
     my($searchfor, $linecounter, $default, $error, $errors);
     my($number, $entr, $attr, $attr2, $result);
     my($rulenum, $ruledir, $ruleaddr, $ruleresult, @ruleset);
@@ -1910,7 +1935,7 @@ sub ReadRuleset {
     }
 
     $RuleScalars{$keyword} = []; # Delete any old inherited rulesets
-    foreach $rulenum ($#ruleset) {
+    foreach $rulenum (0..$#ruleset) {
       #($error, $default) = Store1Rule($ruleset[$rulenum], $rulesfilename,
       ($error, $default) = Store1Rule($ruleset[$rulenum], $keyword,
                                       $rulenum, $rulesettype,
@@ -1924,9 +1949,8 @@ sub ReadRuleset {
     # not a ruleset at all.
     $StaticScalars{$keyword} = $default if $#ruleset==0 && defined($default);
     
-    MailScanner::Log::DieLog("Aborting due to syntax errors in %s.",
-    #                         $rulesfilename) if $errors;
-                             $keyword) if $errors;
+    MailScanner::Log::WarnLog("Found syntax errors in %s.", $keyword)
+      if $errors;
     return;
   }
 
@@ -1937,8 +1961,8 @@ sub ReadRuleset {
 
   $rulesfh = new FileHandle;
   $rulesfh->open("<$rulesfilename")
-    or MailScanner::Log::DieLog('Cannot open ruleset file %s, %s',
-                                $rulesfilename, $!);
+    or MailScanner::Log::WarnLog('Cannot open ruleset file %s, %s',
+                                 $rulesfilename, $!), return;
   flock($rulesfh, $LOCK_SH);
 
   $RuleScalars{$keyword} = []; # Set up empty ruleset
@@ -1958,8 +1982,8 @@ sub ReadRuleset {
   flock($rulesfh, $LOCK_UN);
   $rulesfh->close();
 
-  MailScanner::Log::DieLog("Aborting due to syntax errors in %s.",
-                           $rulesfilename) if $errors;
+  MailScanner::Log::WarnLog("Found syntax errors in %s.", $rulesfilename)
+    if $errors;
 }
 
 
@@ -2017,7 +2041,9 @@ sub Store1Rule {
   #
   my($direction2, $rule2, $fromto2);
   $fromto2 = '';
-  if ($value =~ /^and\s+(\S+)\s+(\S+)(\s+(\S+))?$/i) {
+  #if ($value =~ /^and\s+(\S+)\s+(\S+)(\s+(\S+))?$/i) {
+  # This should fix bug with "and" with multi-value rule results
+  if ($value =~ /^and\s+(\S+)\s+(\S+)(\s+(.+))?$/i) {
     ($direction2, $rule2, $value) = ($1, $2, $4);
     $fromto2 = '';
     if ($direction2 =~ /and/i) {
@@ -2201,12 +2227,14 @@ sub ReadYesNoValue {
   $first = $File{$keyword};
   $isfile = 1 if $first =~ /^\//; # Filenames start with '/'
   $isrules = 1 if $isfile && $first !~ /(txt|html)$/; # Rules ain't called .txt
-  $isrules = 1 if $LDAP && $first =~ /\.RuleSet$/; # LDAP ruleset
+  $isrules = 1 if $LDAP && $first =~ /customi[sz]e|\.RuleSet$/; # LDAP ruleset
 
   # It might be a function name
   if ($first =~ /^&/) {
     $first =~ s/^&//;
-    $CustomFunctions{$keyword} = $first;
+    $CustomFunctionsParams{$keyword} = $CustomFunctions{$keyword} = $first;
+    $CustomFunctions{$keyword} =~ s/\(.*//;
+    $CustomFunctionsParams{$keyword} =~ s/^[^\(]+//;
     return;
   }
 
@@ -2231,7 +2259,7 @@ sub ReadYesNoValue {
       $StaticScalars{$keyword} = $internal;
     } else {
       # It is an invalid value
-      MailScanner::Log::DieLog("Syntax error in line %d, value \"%s\" " .
+      MailScanner::Log::WarnLog("Syntax error in line %d, value \"%s\" " .
         "for %s is not one of allowed values \"%s\"", $LineNos{$keyword},
         $File{$keyword}, $keyword, join("\",\"", keys %values));
     }
@@ -2279,12 +2307,14 @@ sub ReadFileValue {
   $first =~ s/\s+.*$//; # Extract the first word
   $isfile = 1 if $first =~ /^\//; # Filenames start with '/'
   $isrules = 1 if $isfile && $first =~ /rules?$/; # Try to find ruleset
-  $isrules = 1 if $LDAP && $first =~ /\.RuleSet$/; # LDAP ruleset
+  $isrules = 1 if $LDAP && $first =~ /customi[sz]e|\.RuleSet$/; # LDAP ruleset
 
   # It might be a function name
   if ($first =~ /^&/) {
     $first =~ s/^&//;
-    $CustomFunctions{$keyword} = $first;
+    $CustomFunctionsParams{$keyword} = $CustomFunctions{$keyword} = $first;
+    $CustomFunctions{$keyword} =~ s/\(.*//;
+    $CustomFunctionsParams{$keyword} =~ s/^[^\(]+//;
     return;
   }
 
@@ -2309,7 +2339,7 @@ sub ReadFileValue {
       $StaticScalars{$keyword} = $internal;
     } else {
       # It is an invalid value
-      MailScanner::Log::DieLog("Error in line %d, file \"%s\" " .
+      MailScanner::Log::WarnLog("Error in line %d, file \"%s\" " .
         "for %s does not exist (or can not be read)",
         $LineNos{$keyword}, $File{$keyword}, $keyword);
     }
@@ -2356,7 +2386,9 @@ sub ReadCommandValue {
   # It might be a function name
   if ($first =~ /^&/) {
     $first =~ s/^&//;
-    $CustomFunctions{$keyword} = $first;
+    $CustomFunctionsParams{$keyword} = $CustomFunctions{$keyword} = $first;
+    $CustomFunctions{$keyword} =~ s/\(.*//;
+    $CustomFunctionsParams{$keyword} =~ s/^[^\(]+//;
     return;
   }
 
@@ -2380,7 +2412,7 @@ sub ReadCommandValue {
       $StaticScalars{$keyword} = $internal;
     } else {
       # It is an invalid value
-      MailScanner::Log::DieLog("Error in line %d, file \"%s\" " .
+      MailScanner::Log::WarnLog("Error in line %d, file \"%s\" " .
         "for %s does not exist (or can not be read)",
         $LineNos{$keyword}, $File{$keyword}, $keyword);
     }
@@ -2421,12 +2453,14 @@ sub ReadDirValue {
   my($first, $isrules);
   $first = $File{$keyword};
   $isrules = 1 if -f $first; # Rules are files
-  $isrules = 1 if $LDAP && $first =~ /\.RuleSet$/; # LDAP ruleset
+  $isrules = 1 if $LDAP && $first =~ /customi[sz]e|\.RuleSet$/; # LDAP ruleset
 
   # It might be a function name
   if ($first =~ /^&/) {
     $first =~ s/^&//;
-    $CustomFunctions{$keyword} = $first;
+    $CustomFunctionsParams{$keyword} = $CustomFunctions{$keyword} = $first;
+    $CustomFunctions{$keyword} =~ s/\(.*//;
+    $CustomFunctionsParams{$keyword} =~ s/^[^\(]+//;
     return;
   }
 
@@ -2453,7 +2487,7 @@ sub ReadDirValue {
       $StaticScalars{$keyword} = $internal;
     } else {
       # It is an invalid value
-      MailScanner::Log::DieLog("Error in configuration file line %d, " .
+      MailScanner::Log::WarnLog("Error in configuration file line %d, " .
         "directory %s for %s does not exist (or is not readable)",
         $LineNos{$keyword}, $first, $keyword);
     }
@@ -2496,12 +2530,14 @@ sub ReadNumberValue {
   my($first, $isrules);
   $first = $File{$keyword};
   $isrules = 1 if $first !~ /^[\d._-]*$/; # Rules aren't all digits or ._-
-  $isrules = 1 if $LDAP && $first =~ /\.RuleSet$/; # LDAP ruleset
+  $isrules = 1 if $LDAP && $first =~ /customi[sz]e|\.RuleSet$/; # LDAP ruleset
 
   # It might be a function name
   if ($first =~ /^&/) {
     $first =~ s/^&//;
-    $CustomFunctions{$keyword} = $first;
+    $CustomFunctionsParams{$keyword} = $CustomFunctions{$keyword} = $first;
+    $CustomFunctions{$keyword} =~ s/\(.*//;
+    $CustomFunctionsParams{$keyword} =~ s/^[^\(]+//;
     return;
   }
 
@@ -2525,7 +2561,7 @@ sub ReadNumberValue {
       $StaticScalars{$keyword} = $internal;
     } else {
       # It is an invalid value
-      MailScanner::Log::DieLog("Syntax error in line %d, %s for %s " .
+      MailScanner::Log::WarnLog("Syntax error in line %d, %s for %s " .
         "should be a number", $LineNos{$keyword}, $first, $keyword);
     }
   }
@@ -2569,12 +2605,14 @@ sub ReadOtherValue {
   my($first, $isrules);
   $first = $File{$keyword};
   $isrules = 1 if $first =~ /^\// && $first =~ /rules?$/ && -f $first; # Rules are filenames
-  $isrules = 1 if $LDAP && $first =~ /\.RuleSet$/; # LDAP ruleset
+  $isrules = 1 if $LDAP && $first =~ /customi[sz]e|\.RuleSet$/; # LDAP ruleset
 
   # It might be a function name
   if ($first =~ /^&/) {
     $first =~ s/^&//;
-    $CustomFunctions{$keyword} = $first;
+    $CustomFunctionsParams{$keyword} = $CustomFunctions{$keyword} = $first;
+    $CustomFunctions{$keyword} =~ s/\(.*//;
+    $CustomFunctionsParams{$keyword} =~ s/^[^\(]+//;
     return;
   }
 
@@ -2596,6 +2634,8 @@ sub ReadOtherValue {
       # It is a valid value
       #print STDERR "Config: Setting other scalar " . $keyword .
       #             " = $internal\n";
+      # Percent variables need to be substituted here.
+      $internal = DoPercentVars($internal); # %vars% must be defined before use
       $StaticScalars{$keyword} = $internal;
     }
     # Could do some specific syntax checking in here
