@@ -2,7 +2,7 @@
 #   MailScanner - SMTP E-Mail Virus Scanner
 #   Copyright (C) 2002  Julian Field
 #
-#   $Id: MCPMessage.pm,v 1.1.2.8 2004/03/22 11:16:44 jkf Exp $
+#   $Id: MCPMessage.pm,v 1.1.2.16 2004/12/20 14:57:14 jkf Exp $
 #
 #   This program is free software; you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
@@ -49,7 +49,7 @@ install MIME::Decoder::UU 'uuencode';
 use vars qw($VERSION);
 
 ### The package version, both in 1.23 style *and* usable by MakeMaker:
-$VERSION = substr q$Revision: 1.1.2.8 $, 10;
+$VERSION = substr q$Revision: 1.1.2.16 $, 10;
 
 # Is this message spam? Try to build the spam report and store it in
 # the message.
@@ -232,7 +232,7 @@ sub IsMCP {
                               $this->{from}, $todomain, $ReportText);
   }
 
-  return $this->{isspam};
+  return $this->{ismcp};
 }
     
 
@@ -243,21 +243,44 @@ sub HandleMCP {
   my($this, $HamSpam) = @_;
 
   my($actions, $action, @actions, %actions);
+  my(@extraheaders, $actionscopy, $actionkey);
 
   # Get a space-separated list of all the actions
   if ($HamSpam eq 'nonmcp') {
-    $actions = lc(MailScanner::Config::Value('nonmcpactions', $this));
+    $actions = MailScanner::Config::Value('nonmcpactions', $this);
     # Fast bail-out if it's just the simple "deliver" case that 99% of
     # people will use
     return if $actions eq 'deliver';
   } else {
     # It must be spam as it's not ham
     if ($this->{ishighmcp}) {
-      $actions = lc(MailScanner::Config::Value('highscoremcpactions', $this));
+      $actions = MailScanner::Config::Value('highscoremcpactions', $this);
     } else {
-      $actions = lc(MailScanner::Config::Value('mcpactions', $this));
+      $actions = MailScanner::Config::Value('mcpactions', $this);
     }
   }
+
+  # Find all the bits in quotes, with their spaces
+  $actionscopy = $actions;
+  #print STDERR "Actions = \'$actions\'\n";
+  while ($actions =~ s/\"([^\"]+)\"//) {
+    $actionkey = $1;
+    push @extraheaders, $actionkey;
+    MailScanner::Log::WarnLog("Syntax error in \"header\" action in MCP " .
+                              "actions, missing \":\" in %s", $actionkey)
+      unless $actionkey =~ /:/;
+  }
+  @{$this->{extramcpheaders}} = @extraheaders;
+  $actions = lc($actions);
+  $actions =~ s/^\s*//;
+  $actions =~ s/\s*$//;
+  $actions =~ s/\s+/ /g;
+  #print STDERR "Actions after = \'$actions\'\n";
+  #print STDERR "Extra headers are \"" . join(',',@extraheaders) . "\"\n";
+
+  MailScanner::Log::WarnLog('Syntax error: missing " in MCP actions %s',
+                            $actionscopy) if $actions =~ /\"/;
+
   $actions =~ tr/,//d; # Remove all commas in case they put any in
   @actions = split(" ", $actions);
 
@@ -315,7 +338,8 @@ sub HandleMCP {
       if $HamSpam eq 'spam' && MailScanner::Config::Value('logmcp');
 
     # Mark the message as deleted, so it won't get delivered
-    $this->{deleted} = 1;
+    #$this->{deleted} = 1;
+    $this->{dontdeliver} = 1; # Don't clean or deliver this message, just drop
     # Mark this message as not being delivered by MCP, for later spam filtering
     $this->{mcpdelivering} = 0;
   }
@@ -365,7 +389,11 @@ sub HandleMCP {
     }
     #print STDERR "Storing spam to $spamdir/" . $this->{id} . "\n";
     umask $global::MS->{quar}->{fileumask};
-    $this->{store}->CopyEntireMessage($this, $spamdir, $this->{id});
+    my @paths = $this->{store}->CopyEntireMessage($this, $spamdir, $this->{id});
+    # Remember where we have stored the mcp in an archive, so we never
+    # archive infected messages.
+    push @{$this->{spamarchive}}, @paths;
+    chown $uid, $gid, "$spamdir/" . $this->{id}; # Harmless if this fails
   }
   umask 0077; # Safety net
 
@@ -463,17 +491,70 @@ sub HandleMCPBounce {
     # Boring untainting again...
     /(.*)/;
     $line = eval "\"$1\"";
-    $emailmsg .= $line . "\n";
+    $emailmsg .= MailScanner::Config::DoPercentVars($line) . "\n";
   }
   $messagefh->close();
 
-  # Send the message to the spam sender, but ensure the envelope
-  # sender address is "<>" so that it can't be bounced.
-  $global::MS->{mta}->SendMessageString($this, $emailmsg, '<>')
-    or MailScanner::Log::WarnLog("Could not send sender MCP bounce, %s", $!);
+  if (MailScanner::Config::Value('bouncemcpasattachment', $this)) {
+    $this->HandleMCPBounceAttachment($emailmsg);
+  } else {
+    # Send the message to the spam sender, but ensure the envelope
+    # sender address is "<>" so that it can't be bounced.
+    $global::MS->{mta}->SendMessageString($this, $emailmsg, '<>')
+      or MailScanner::Log::WarnLog("Could not send sender MCP bounce, %s", $!);
+  }
 }
 
 
+# Like encapsulating and sending a message to the recipient, take the
+# passed text as the text and headers of an email message and attach
+# the original message as an rfc/822 attachment.
+sub HandleMCPBounceAttachment {
+  my($this, $plaintext) = @_;
+
+  my $parser = MIME::Parser->new;
+  my $explodeinto = $global::MS->{work}->{dir} . '/' . $this->{id};
+  #print STDERR "Extracting MCP bounce message into $explodeinto\n";
+  my $filer  = MIME::Parser::FileInto->new($explodeinto);
+  $parser->filer($filer);
+
+  my $bounce = eval { $parser->parse_data(\$plaintext) };
+  if (!$bounce) {
+    MailScanner::Log::WarnLog("Cannot parse MCP bounce report, %s", $!);
+    return;
+  }
+  #print STDERR "Successfully parsed bounce report\n";
+
+  # Now make it multipart and push the report into a child
+  $bounce->make_multipart('report');
+
+  # Now turn the original message into a string and attach it
+  my(@original);
+  #my $original = $this->{entity}->stringify;
+  @original = $global::MS->{mta}->OriginalMsgHeaders($this, "\n");
+  push(@original, "\n");
+  $this->{store}->ReadBody(\@original, MailScanner::Config::Value(
+                                          'maxspamassassinsize'));
+
+  $bounce->add_part(MIME::Entity->build(Type        => 'message/rfc822',
+                                        Disposition => 'attachment',
+                                        Top         => 0,
+                                        'X-Mailer'  => undef,
+                                        Data        => \@original));
+
+  # Stringify the message and send it -- this could be VERY large!
+  my $bouncetext = $bounce->stringify;
+  #print STDERR "Spam bounce message is this:\n$bouncetext";
+  if ($bouncetext) {
+    $global::MS->{mta}->SendMessageString($this, $bouncetext, '<>')
+      or MailScanner::Log::WarnLog(
+           "Could not send sender MCP bounce attachment, %s", $!);
+  } else {
+    MailScanner::Log::WarnLog(
+      "Failed to create sender MCP bounce attachment, %s", $!);
+  }
+}
+
 
 # We want to send a message to the recipient saying that their spam
 # mail has not been delivered.
@@ -550,7 +631,7 @@ sub HandleMCPNotify {
     # Boring untainting again...
     /(.*)/;
     $line = eval "\"$1\"";
-    $emailmsg .= $line . "\n";
+    $emailmsg .= MailScanner::Config::DoPercentVars($line) . "\n";
   }
   $messagefh->close();
 
