@@ -2,7 +2,7 @@
 #   MailScanner - SMTP E-Mail Virus Scanner
 #   Copyright (C) 2002  Julian Field
 #
-#   $Id: MessageBatch.pm,v 1.64.2.23 2004/04/28 14:00:29 jkf Exp $
+#   $Id: MessageBatch.pm,v 1.64.2.28 2004/12/12 16:17:24 jkf Exp $
 #
 #   This program is free software; you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
@@ -44,7 +44,7 @@ use DirHandle;
 use vars qw($VERSION);
 
 ### The package version, both in 1.23 style *and* usable by MakeMaker:
-$VERSION = substr q$Revision: 1.64.2.23 $, 10;
+$VERSION = substr q$Revision: 1.64.2.28 $, 10;
 
 my $maxcleanbytes    = 0;
 my $maxcleanmessages = 0;
@@ -426,7 +426,6 @@ sub Explode {
 sub VirusScan {
   my $this = shift;
 
-
   # Call them with the scanning settings
   MailScanner::Log::InfoLog("Virus and Content Scanning: Starting");
   my $viruses = MailScanner::SweepViruses::ScanBatch($this, 'scan');
@@ -486,6 +485,24 @@ sub ReportBadMessages {
 }
 
 
+# Remove any infected spam or mcp from their archives. We have saved
+# all the spam+mcp archive places we stored this message, so go delete
+# the dirs and all the files in each one.
+sub RemoveInfectedSpam {
+  my $this = shift;
+
+  my($id, $message);
+
+  while(($id, $message) = each %{$this->{messages}}) {
+    #print STDERR "Message is infected\n" if $message->{infected};
+    next unless $message->{infected};
+    next unless
+      MailScanner::Config::Value('keepspamarchiveclean', $message) =~ /1/;
+    #print STDERR "Deleting " . join(',',@{$message->{spamarchive}}) . "\n";
+    unlink @{$message->{spamarchive}};
+  }
+}
+
 # Set up the entity2file and entity2parent hashes in every message
 sub CreateEntitiesHelpers {
   my $this = shift;
@@ -519,7 +536,7 @@ sub PrintFilenames {
   my($fnames, @filenames);
 
   while(($id, $message) = each %{$this->{messages}}) {
-    next if $message->{deleted};
+    #next if $message->{deleted};
     #print STDERR "Message $id has filenames ";
     @filenames = keys %{$message->{file2entity}};
     #print STDERR join(", ", @filenames) . "\n";
@@ -559,7 +576,7 @@ sub Clean {
   my($id, $message);
 
   while(($id, $message) = each %{$this->{messages}}) {
-    next if $message->{deleted};
+    next if $message->{deleted} || $message->{dontdeliver};
     #print STDERR "\nCleaning message $id\n";
     $message->Clean();
   }
@@ -589,6 +606,8 @@ sub QuarantineInfections {
   while(($id, $message) = each %{$this->{messages}}) {
     next if $message->{deleted};
     next unless $message->{infected};
+    next if $message->{silent} && !$message->{noisy} &&
+            MailScanner::Config::Value('quarantinesilent', $message) !~ /1/;
     next unless MailScanner::Config::Value('quarantineinfections', $message)
                 =~ /1/;
     #print STDERR "Quarantining infections for $id\n";
@@ -605,7 +624,7 @@ sub SignUninfected {
   my($id, $message);
 
   while(($id, $message) = each %{$this->{messages}}) {
-    next if $message->{deleted};
+    next if $message->{deleted} || $message->{dontdeliver};
     $message->SignUninfected()
       if MailScanner::Config::Value('signcleanmessages', $message) &&
          !$message->{infected};
@@ -624,7 +643,7 @@ sub DeliverUninfected {
   my($id, $message, @messages);
 
   while(($id, $message) = each %{$this->{messages}}) {
-    next if $message->{deleted};
+    next if $message->{deleted} || $message->{dontdeliver};
     #print STDERR "Possibly delivering uninfected message $id\n";
     next if $message->{infected};
     #print STDERR "Delivering uninfected message $id\n";
@@ -688,7 +707,8 @@ sub DeliverOrDeleteSilent {
   my($id, $message, @messages);
 
   while(($id, $message) = each %{$this->{messages}}) {
-    next if !$message->{silent} || $message->{noisy} || $message->{deleted};
+    next if !$message->{silent}  || $message->{noisy} ||
+             $message->{deleted} || $message->{dontdeliver};
     #MailScanner::Log::WarnLog("Deliversilent for %s is %s", $message->{id},
     #                MailScanner::Config::Value('deliversilent', $message));
     if (MailScanner::Config::Value('deliversilent', $message)) {
@@ -716,7 +736,7 @@ sub DeliverCleaned {
   my($id, $message, @messages);
 
   while(($id, $message) = each %{$this->{messages}}) {
-    next if $message->{deleted};
+    next if $message->{deleted} || $message->{dontdeliver};
     $message->DeliverCleaned();
     #print STDERR "Deleting cleaned message " . $message->{id} . "\n";
     # BUGFIX: JKF $message->{deleted} = 1;
@@ -783,7 +803,7 @@ sub WarnLocalPostmaster {
 
   my($id, $message, $counter);
   my(%notices, $notice, %headers, %signatures, $email);
-  my(@posties, $posties, $postie, $sig);
+  my(@posties, $posties, $postie, $sig, %reasons, $reasons, $thisreason);
 
   # Create all the email messages
   $counter = 0;
@@ -797,6 +817,21 @@ sub WarnLocalPostmaster {
     foreach $postie (@posties) {
       $headers{$postie} = $message->CreatePostmasterHeaders($postie)
         unless $headers{$postie};
+
+      # Change the subject to include the problem types
+      %reasons = ();
+      $reasons = "";
+      $reasons{virusinfected}     = 1 if $message->{virusinfected};
+      $reasons{filenameinfected}  = 1 if $message->{nameinfected};
+      $reasons{otherinfected}     = 1 if $message->{otherinfected};
+      $reasons{passwordprotected} = 1 if $message->{passwordprotected};
+      foreach $thisreason (sort keys %reasons) {
+        $reasons .= " : " if $reasons ne "";
+        $reasons .= MailScanner::Config::LanguageValue($message, "notice" .
+                                                       $thisreason);
+      }
+      $headers{$postie} =~ s/\nSubject:.*?\n/\nSubject: $reasons\n/si;
+
       $notices{$postie} .= $message->CreatePostmasterNotice();
       unless ($signatures{$postie}) {
         $sig = MailScanner::Config::Value('noticesignature', $message);
@@ -808,8 +843,10 @@ sub WarnLocalPostmaster {
   }
   while(($postie,$notice) = each %notices) {
     $email = $headers{$postie} . "\n" .
-      MailScanner::Config::LanguageValue($message, 'noticeheading') . ":\n" .
-      $notices{$postie} . "\n" . $signatures{$postie} . "\n";
+      #MailScanner::Config::LanguageValue($message, 'noticeheading') . ":\n" .
+      #$notices{$postie} . "\n" . $signatures{$postie} . "\n";
+      MailScanner::Config::LanguageValue($message, 'noticeprefix') . ": " .
+      $reasons . "\n" . $notices{$postie} . "\n" . $signatures{$postie} . "\n";
 
     $global::MS->{mta}->SendMessageString(undef, $email, $postie) or
       MailScanner::Log::WarnLog("Could not notify postmaster from $postie, %s",
@@ -836,6 +873,7 @@ sub DisinfectAndDeliver {
   # such as DoS attacks.
   while(($id, $message) = each %{$this->{messages}}) {
     if ($message->{deleted} ||
+        $message->{dontdeliver} ||
         $message->{cantparse} ||
         $message->{badtnef} ||
         $message->{nameinfected} ||
@@ -917,7 +955,7 @@ sub StripHTML {
 
   my($id, $message);
   while(($id, $message) = each %{$this->{messages}}) {
-    next if $message->{deleted};
+    next if $message->{deleted} || $message->{dontdeliver};
     next unless $message->{needsstripping};
     $message->StripHTML();
   }
@@ -930,7 +968,9 @@ sub DisarmHTML {
 
   my($id, $message);
   while(($id, $message) = each %{$this->{messages}}) {
-    next if $message->{deleted};
+    #print STDERR "In MessageBatch, tags are " .
+    #             $message->{tagstoconvert} . "\n";
+    next if $message->{deleted} || $message->{dontdeliver};
     next unless $message->{tagstoconvert};
     $message->DisarmHTML();
   }
@@ -944,7 +984,7 @@ sub Encapsulate {
 
   my($id, $message);
   while(($id, $message) = each %{$this->{messages}}) {
-    next if $message->{deleted};
+    next if $message->{deleted} || $message->{dontdeliver};
     next unless $message->{needsencapsulating};
     $message->EncapsulateMessage();
   }
