@@ -2,7 +2,7 @@
 #   MailScanner - SMTP E-Mail Virus Scanner
 #   Copyright (C) 2002  Julian Field
 #
-#   $Id: PFDiskStore.pm,v 1.1.2.25 2004/03/31 00:47:32 jkf Exp $
+#   $Id: PFDiskStore.pm,v 1.1.2.29 2004/12/14 16:29:35 jkf Exp $
 #
 #   This program is free software; you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
@@ -46,7 +46,7 @@ use MailScanner::Config;
 use vars qw($VERSION @DeletesPending);
 
 ### The package version, both in 1.23 style *and* usable by MakeMaker:
-$VERSION = substr q$Revision: 1.1.2.25 $, 10;
+$VERSION = substr q$Revision: 1.1.2.29 $, 10;
 
 # List of pending delete operations so we can clear up properly when killed
 @DeletesPending = ();
@@ -59,7 +59,7 @@ $VERSION = substr q$Revision: 1.1.2.25 $, 10;
 #use vars qw($VERSION);
 #
 #### The package version, both in 1.23 style *and* usable by MakeMaker:
-#$VERSION = substr q$Revision: 1.1.2.25 $, 10;
+#$VERSION = substr q$Revision: 1.1.2.29 $, 10;
 
 # Attributes are
 #
@@ -222,7 +222,14 @@ sub WriteHeader {
 
   ($predata, $HeaderStartPos) = MailScanner::Sendmail::PreDataString($message);
   #print STDERR "Writing predata \"$predata\"\n";
+  #print STDERR "Length of predata is " . length($predata) . "\n";
+  #print STDERR "Before writing predata we are at " . $Tf->tell . "\n";
   $Tf->print($predata);
+  #print STDERR "Predata is \"$predata\"\n";
+  #print STDERR "After writing predata, file is at " . $Tf->tell() . "\n";
+
+  # Flush the filehandle to save duplicate writes in some Perls
+  $Tf->flush();
 
   #print STDERR "In WriteHeader, header starts at $HeaderStartPos\n";
 
@@ -242,6 +249,7 @@ sub WriteHeader {
       }
       $b->Done();
     }
+    $Tf->flush();
   } elsif ($this->{body}[0] eq "MIME" ) {
     my ($type, $id, $entity, $outq)= @{$this->{body}};
     # This needs re-writing, as we need to massage every line
@@ -272,19 +280,27 @@ sub WriteHeader {
         or MailScanner::Log::WarnLog("WriteMIMEBody to %s possibly failed, %s",
                                      $tfile, $!);
       $pipe->close();
+      $Tf->flush();
       exit;
     }
   }
-  my $PostStartPos = tell $Tf;
-  #print STDERR "Post data starts at $PostStartPos\n";
+  my($PostStartPos, $HeaderLength);
+  $PostStartPos = tell $Tf;
+  #print STDERR "\n\nPost data starts at $PostStartPos\n";
 
   $Tf->print(MailScanner::Sendmail::PostDataString($message));
+  #print STDERR "PostDataString = \"" . MailScanner::Sendmail::PostDataString($message) . "\"\n";
+  $Tf->flush();
 
   # Now over-write the length records in the 1st record and in the M record
   seek $Tf, 2, 0;
 
-  #print STDERR "\nC record contains " . $PostStartPos-$HeaderStartPos .
-  #             " and " . $HeaderStartPos . "\n";
+  #print STDERR "\n\nPostStartPos = \"$PostStartPos\"\n\n";
+  #print STDERR "\n\nHeaderStartPos = \"$HeaderStartPos\"\n\n";
+  $HeaderLength = ($PostStartPos-$HeaderStartPos);
+
+  #print STDERR "\nC record contains headerlength " . $HeaderLength .
+  #             " and headerstartpos " . $HeaderStartPos . "\n";
 
   # Count the number of recipients in the metadata now
   my ($recipcounter, $record);
@@ -295,11 +311,12 @@ sub WriteHeader {
   }
 
 
-  printf $Tf "%15ld %15ld %15ld", $PostStartPos-$HeaderStartPos,
-             $HeaderStartPos, $recipcounter;
+  #print STDERR "Writing Data length = " . $PostStartPos-$HeaderStartPos . " Header start = $HeaderStartPos Recip Counter = $recipcounter\n";
+  printf $Tf "%15ld %15ld %15ld", $HeaderLength, $HeaderStartPos, $recipcounter;
   printf $Tf " %15ld", $message->{PostfixQmgrOpts}
     if $message->{PostfixQmgrOpts} ne "";
   seek $Tf, 0, 0;
+  #print STDERR "Seeked to start of file\n";
   # Find the M record
   my($MPos, $type, $data);
   $MPos = 0;
@@ -307,6 +324,7 @@ sub WriteHeader {
   while(defined($type) && $type ne 'M') {
     $MPos = tell $Tf;
     ($type, $data) = MailScanner::Sendmail::ReadRecord($Tf);
+    #print STDERR "Skipping over record \"$data\"\n";
   }
   MailScanner::Log::WarnLog("Corrupt queue output file") unless defined($type);
   unless ($MailScanner::Postfix::DataStructure > 0) {
@@ -444,6 +462,8 @@ sub WriteMIMEBody {
 # JKF and no directory components.
 # JKF The hashing directory components will be extracted when needed.
 # JKF Is now much simpler, just calls the functions that do the job already.
+# $targetfile can be 'message' or undef which indicates we are storing in the
+# main quarantine, not in the outgoing mail dir.
 sub CopyEntireMessage {
   my $this = shift;
   my($message, $targetdir, $targetfile, $uid, $gid, $changeowner) = @_;
@@ -451,13 +471,15 @@ sub CopyEntireMessage {
   #print STDERR "Copying to $targetdir $targetfile\n";
   if (MailScanner::Config::Value('storeentireasdfqf')) {
     #print STDERR "Copying to dir $targetdir\n";
-    $this->CopyToDir($targetdir, $uid, $gid, $changeowner);
+    return ($this->CopyToDir($targetdir, $targetfile, $uid, $gid,
+                             $changeowner));
   } else {
     #print STDERR "Copying to file $targetdir/$targetfile\n";
     my $target = new IO::File "$targetdir/$targetfile", "w";
     MailScanner::Log::WarnLog("writing to $targetdir/$targetfile: $!")
       if not defined $target;
     $this->WriteEntireMessage($message, $target);
+    return $targetdir . '/' . $targetfile;
   }
 }
 
@@ -525,12 +547,25 @@ sub WriteEntireMessage {
 }
 
 # Copy a hdfile to a directory
+# The Postfix version of this needs to know the destination filename too
+# so it can work out whether to use the hdpath as the destination filename
+# (which just has the 10 hex digits in it) or the message id, which has
+# the random number added to the end of it too.
 sub CopyToDir {
-  my($this,$dir,$uid,$gid,$changeowner) = @_;
-  my $hdpath = $this->{hdpath};
-  my $hdfile = basename($hdpath);
+  my($this,$dir,$file,$uid,$gid,$changeowner) = @_;
+  my($hdpath, $hdfile);
+  $hdpath = $this->{hdpath};
+  if ($file && $file ne 'message') {
+    #$hdfile = basename($hdpath);
+    $hdfile = $file;
+  } else {
+    # We weren't passed a sensible filename, so work one out for ourselves.
+    $hdfile = $this->{id}; #basename($hdpath);
+print STDERR "hdfile = " . $this->{id} . "\n";
+  }
   copy($hdpath, "$dir/$hdfile");
   chown $uid, $gid, "$dir/$hdfile" if $changeowner;
+  return "$dir/$hdfile";
 }
 
 package Body;
