@@ -2,7 +2,7 @@
 #   MailScanner - SMTP E-Mail Virus Scanner
 #   Copyright (C) 2002  Julian Field
 #
-#   $Id: Postfix.pm,v 1.1.2.42 2004/03/31 00:47:33 jkf Exp $
+#   $Id: Postfix.pm,v 1.1.2.53 2004/12/18 12:25:46 jkf Exp $
 #
 #   This program is free software; you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
@@ -41,7 +41,7 @@ use DirHandle;
 use vars qw($VERSION);
 
 ### The package version, both in 1.23 style *and* usable by MakeMaker:
-$VERSION = substr q$Revision: 1.1.2.42 $, 10;
+$VERSION = substr q$Revision: 1.1.2.53 $, 10;
 
 # Command-line options you need to give to sendmail to sensibly process a
 # message that is piped to it. Still need to add "-f" for specifying the
@@ -237,6 +237,7 @@ sub new {
     #$id =~ /^(.)(.)(.+)$/;
     #($dir1, $dir2, $file) = ($1,$2,$3);
     #return "$dir1/$dir2/$file";
+    $id =~ s/\.[^.]+$//;
     return "$id";
   }
 
@@ -304,13 +305,13 @@ sub new {
     MailScanner::Log::WarnLog("Syntax error in Postfix queue file, didn't " .
                               "start with a C record") unless $rectype eq 'C';
     $recdata =~ /^([0-9 ]{15}) ([0-9 ]{15}) ([0-9 ]{15})( ([0-9 ]{15}))?$/;
-    ($MsgContSize, $DataOffset, $NumRecips) = ($1, $2, $3);
+    ($MsgContSize, $DataOffset, $NumRecips) = ($1+0, $2+0, $3+0);
 
-    # If $4 is set then we have a new data structure in the file
+    # If $5 is set then we have a new data structure in the file
     $MailScanner::Postfix::DataStructure = 0;
-    if ($4 ne "") {
+    if ($5 ne "") {
       $MailScanner::Postfix::DataStructure = 1;
-      $message->{PostfixQmgrOpts} = $4;
+      $message->{PostfixQmgrOpts} = $5+0;
     }
 
     $MsgContSize =~ s/^\s*//;
@@ -379,7 +380,7 @@ sub new {
       }
       last if $rectype eq 'N' && $recdata eq "";
       push @{$message->{headers}}, $recdata; # Headers have no leading N
-      if ($recdata =~ /^Subject:\s+(\S.*)$/i) {
+      if ($recdata =~ /^Subject:\s*(\S.*)?$/i) {
         $message->{subject} = $1;
         next;
       }
@@ -439,6 +440,13 @@ sub new {
       }
     }
       
+    # Remove all the duplicates from ->{to}
+    my %uniqueto;
+    foreach (@{$message->{to}}) {
+      $uniqueto{$_} = 1;
+    }
+    @{$message->{to}} = keys %uniqueto;
+
     # We now have all the pre-message records followed by the M record
     # followed by the post-message records including the X record and the
     # terminating E record. We can add recipient R records just before
@@ -680,7 +688,7 @@ sub new {
       return;
     }
 
-    $message->{metadata}[$oldlocation] =~ s/^N$key\s+/N$key $newvalue$sep/i;
+    $message->{metadata}[$oldlocation] =~ s/^N$key\s*/N$key $newvalue$sep/i;
   }
 
   sub TextStartsHeader {
@@ -756,13 +764,23 @@ sub new {
     my $this = shift;
     my($message, @recips) = @_;
 
+    # Remove all the duplicates @recips
+    my %uniqueto;
+    foreach (@recips) {
+      $uniqueto{$_} = 1;
+    }
+    @recips = keys %uniqueto;
+
     my $totallines = @{$message->{metadata}};
 
     foreach (@recips) {
       s/^/R/;
     }
 
-    splice @{$message->{metadata}}, $totallines-2, 0, @recips;
+    # Changed 2 to 1 in next line for Postfix 2.1
+    splice @{$message->{metadata}}, $totallines-1, 0, @recips;
+    #print STDERR "AddRecipients: " . join(',',@recips) . "\n";
+    #print STDERR "metadata is \"" . join("\n", @{$message->{metadata}}) . "\n";
   }
 
   # Delete the original recipients from the message. We'll add some
@@ -922,6 +940,22 @@ sub new {
   }
 
 
+  # Append, add or replace a given header with a given value.
+  sub AddMultipleHeaderName {
+    my $this = shift;
+    my($message, $headername, $headervalue, $separator) = @_;
+
+    my($multiple) = MailScanner::Config::Value('multipleheaders', $message);
+    $this->AppendHeader ($message, $headername, $headervalue, $separator)
+      if $multiple eq 'append';
+
+    $this->AddHeader    ($message, $headername, $headervalue)
+      if $multiple eq 'add';
+
+    $this->ReplaceHeader($message, $headername, $headervalue)
+      if $multiple eq 'replace';
+  }
+
   # Append, add or replace a given header with a given value.
   sub AddMultipleHeader {
     my $this = shift;
@@ -1102,7 +1136,11 @@ sub FindHashDirDepth {
       $invalidfiles = "";
 
       # Loop through each of the inq directories
-      foreach $queuedirname (@queuedirnames) {
+      # Patch to combat starving in emergency queue mode
+      # foreach $queuedirname (@queuedirnames) {
+      my @aux_queuedirnames=@queuedirnames;
+      while( defined($queuedirname=splice(@aux_queuedirnames,
+            ($UnsortedBatchesLeft<=0 ? 0 :int(rand(@aux_queuedirnames))),1))) {
         #print STDERR "Scanning dir $queuedirname\n";
         unless (chdir $queuedirname) {
           MailScanner::Log::WarnLog("Cannot cd to dir %s to read messages, %s",
@@ -1220,14 +1258,20 @@ sub FindHashDirDepth {
           if $MailScanner::SMDiskStore::HashDirDepth == 1 &&
              $file =~ /^(.*)\/(.)\/([^\/]+)$/;
         next unless $file =~ /$mta->{HDFileRegexp}/;
-        $id = $1;
+        # Put the real message id in $idorig and the unique name in $id
+        # JKF Add a dot followed by a random number to try to get a unique
+        # JKF filename, as Postfix re-uses filenames too often.
+        my $idtemp = $1;
+        my $id = $idtemp . sprintf(".%05X", int(rand 1000000)+1);
+        my $idorig = $idtemp;
 
         #print STDERR "Adding $id to batch\n";
 
         # Lock and read the qf file. Skip this message if the lock fails.
         $newmessage = MailScanner::Message->new($id, $queuedirname);
         if ($newmessage eq 'INVALID') {
-          $invalidfiles .= "$id ";
+          #$invalidfiles .= "$id ";
+          $invalidfiles .= "$idorig ";
           next;
         }
         next unless $newmessage;
@@ -1236,7 +1280,7 @@ sub FindHashDirDepth {
         #print STDERR "Added $id to batch\n";
         $batchempty = 0;
 
-        if (MailScanner::Config::Value("virusscan", $newmessage)) {
+        if (MailScanner::Config::Value("virusscan", $newmessage) =~ /1/) {
           $newmessage->NeedsScanning(1);
           $DirtyMsgs++;
           $DirtyBytes += $newmessage->{size};
