@@ -2,7 +2,7 @@
 #   MailScanner - SMTP E-Mail Virus Scanner
 #   Copyright (C) 2002  Julian Field
 #
-#   $Id: SA.pm,v 1.35.2.36 2004/04/14 09:38:21 jkf Exp $
+#   $Id: SA.pm,v 1.35.2.47 2004/12/14 16:29:36 jkf Exp $
 #
 #   This program is free software; you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
@@ -44,7 +44,7 @@ use POSIX qw(:signal_h); # For Solaris 9 SIG bug workaround
 use vars qw($VERSION);
 
 ### The package version, both in 1.23 style *and* usable by MakeMaker:
-$VERSION = substr q$Revision: 1.35.2.36 $, 10;
+$VERSION = substr q$Revision: 1.35.2.47 $, 10;
 
 # Attributes are
 #
@@ -320,13 +320,28 @@ sub Checks {
   # Construct the array of lines of the header and body of the message
   # JKF 30/1/2002 Don't chop off the line endings. Thanks to Andreas Piper
   #               for this.
-  @WholeMessage = $global::MS->{mta}->OriginalMsgHeaders($message, "\n");
+  # For SpamAssassin 3 we add the "EnvelopeFrom" header to make SPF work
+  my $fromheader = MailScanner::Config::Value('envfromheader', $message);
+  $fromheader =~ s/:$//;
+
+  # Build a list of all the headers, so we can remove any $fromheader that
+  # is already in there.
+  my @SAheaders = $global::MS->{mta}->OriginalMsgHeaders($message, "\n");
+  @SAheaders = grep !/^$fromheader\:/i, @SAheaders;
+  @SAheaders = grep !/^\s*$/, @SAheaders; # ditch blank lines
+
+  push(@WholeMessage, $fromheader . ': ' . $message->{from} . "\n")
+    if $fromheader;
+
+  #push(@WholeMessage, $global::MS->{mta}->OriginalMsgHeaders($message, "\n"));
+  push(@WholeMessage, @SAheaders);
   #print STDERR "Headers are : " . join(', ', @WholeMessage) . "\n";
   unless (@WholeMessage) {
     flock($RebuildLockH, $LOCK_UN) if $BayesIsLocked;
     $RebuildLockH->close() if $MailScanner::SA::DoingBayesRebuilds;
     return (0,0, MailScanner::Config::LanguageValue($message, 'sanoheaders'), 0);
   }
+
   push(@WholeMessage, "\n");
   $message->{store}->ReadBody(\@WholeMessage, $maxsize);
 
@@ -423,13 +438,30 @@ sub SAForkAndTest {
       }
     }
     # Get the autolearn status
-    if (!defined $spamness->{auto_learn_status}) {
-         $AutoLearn = "no";
-    } elsif ($spamness->{auto_learn_status}) {
-          $AutoLearn = "spam";
+    if ($SAversion<3) {
+      # Old code
+      if (!defined $spamness->{auto_learn_status}) {
+        $AutoLearn = "no";
+      } elsif ($spamness->{auto_learn_status}) {
+        $AutoLearn = "spam";
+      } else {
+        $AutoLearn = "not spam";
+      }
     } else {
-          $AutoLearn = "not spam";
+      # New code
+      $spamness->learn();
+      $AutoLearn = $spamness->{auto_learn_status};
+      $AutoLearn = 'no' if $AutoLearn eq 'failed' || $AutoLearn eq "";
+      $AutoLearn = 'not spam' if $AutoLearn eq 'ham';
     }
+    #if (!defined $spamness->{auto_learn_status} || $spamness->{auto_learn_status} eq 'no') {
+    #     $AutoLearn = "no";
+    #} elsif ($spamness->{auto_learn_status}) {
+    #      $AutoLearn = "spam";
+    #} else {
+    #      $AutoLearn = "not spam";
+    #}
+    #sleep 30 if rand(3)>=2.0;
     print $pipe $AutoLearn . "\n";
 
     print $pipe $HitList . "\n";
@@ -469,6 +501,8 @@ sub SAForkAndTest {
     # Roll the queue along one
     $SAsuccessqsum += (shift @SAsuccessqueue)?1:-1
           if @SAsuccessqueue>$queuelength;
+    #print STDERR "Success: sum = $SAsuccessqsum\n";
+    $SAsuccessqsum = 0 if $SAsuccessqsum<0;
   };
   alarm 0;
   # Workaround for bug in perl shipped with Solaris 9,
@@ -510,6 +544,8 @@ sub SAForkAndTest {
       # Roll the queue along one
       $SAsuccessqsum += (shift @SAsuccessqueue)?1:-1
         if @SAsuccessqueue>$queuelength;
+      #print STDERR "Failure: sum = $SAsuccessqsum\n";
+      $SAsuccessqsum = 0 if $SAsuccessqsum<0;
 
       if ($SAsuccessqsum>$maxfailures && @SAsuccessqueue>=$queuelength) {
         MailScanner::Log::WarnLog("SpamAssassin timed out (with no network" .
