@@ -2,7 +2,7 @@
 #   MailScanner - SMTP E-Mail Virus Scanner
 #   Copyright (C) 2002  Julian Field
 #
-#   $Id: SweepContent.pm,v 1.17.2.13 2004/03/18 11:21:46 jkf Exp $
+#   $Id: SweepContent.pm,v 1.17.2.19 2004/11/04 14:34:30 jkf Exp $
 #
 #   This program is free software; you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
@@ -41,7 +41,7 @@ use DirHandle;
 use vars qw($VERSION);
 
 ### The package version, both in 1.23 style *and* usable by MakeMaker:
-$VERSION = substr q$Revision: 1.17.2.13 $, 10;
+$VERSION = substr q$Revision: 1.17.2.19 $, 10;
 
 # Attributes are
 #
@@ -81,8 +81,15 @@ sub ScanBatch {
 
   while(($id, $message) = each %{$batch->{messages}}) {
     next if $message->{deleted};
+
     $ent = $message->{entity};
 
+    #
+    # Do the partial and external checks even if they don't want
+    # dangerous content scanning, as they directly affect our ability
+    # to scan for viruses.
+    #
+
     # Search for multipart/partial messages. This is entity-based as
     # the last part of the message (which is what is split into the
     # next message) probably won't have a filename.
@@ -100,6 +107,19 @@ sub ScanBatch {
                                 'fragmented message section in %s', $id);
     }
 
+    # Search for message/external-body messages. This is entity-based
+    # as, almost by definition, we won't have the filename worked out
+    # for the file that is external.
+    if ((! MailScanner::Config::Value('allowexternal', $message)) &&
+        FindExternalBody($message, $ent)) {
+      $counter++;
+      MailScanner::Log::WarnLog('Content Checks: Detected and rejected ' .
+                                'external message body in %s', $id);
+    }
+
+    # Do the remaining checks if any recipient wants dangerous content checking
+    next unless MailScanner::Config::Value('dangerscan', $message) =~ /1/;
+
     # Go through the attachments and remove any that are bigger than this
     # user/domain/site is allowed.
     my $maxmessagesize = MailScanner::Config::Value('maxmessagesize', $message);
@@ -122,27 +142,32 @@ sub ScanBatch {
     my $objectvalue = MailScanner::Config::Value('allowobjecttags', $message);
     my $formvalue   = MailScanner::Config::Value('allowformtags',   $message);
     my $scriptvalue = MailScanner::Config::Value('allowscripttags', $message);
+    my $webbugvalue = MailScanner::Config::Value('allowwebbugtags', $message);
     my $allowiframes = 0;
     my $allowobjects = 0;
     my $allowforms   = 0;
     my $allowscripts = 0;
+    my $allowwebbugs = 0;
     my $convertiframes = 0;
     my $convertobjects = 0;
     my $convertforms   = 0;
     my $convertscripts = 0;
+    my $convertwebbugs = 0;
     # Allow the tags only if everyone allows them
     $allowiframes = 1 if $iframevalue =~ /^[1\s]+$/;
     $allowobjects = 1 if $objectvalue =~ /^[1\s]+$/;
     $allowforms   = 1 if $formvalue   =~ /^[1\s]+$/;
     $allowscripts = 1 if $scriptvalue =~ /^[1\s]+$/;
+    $allowwebbugs = 1 if $webbugvalue =~ /^[1\s]+$/;
     # Convert the tags if no-one blocks them and someone converts them
     $convertiframes = 1 if $iframevalue !~ /0/ && $iframevalue =~ /convert/i;
     $convertobjects = 1 if $objectvalue !~ /0/ && $objectvalue =~ /convert/i;
     $convertforms   = 1 if $formvalue   !~ /0/ && $formvalue   =~ /convert/i;
     $convertscripts = 1 if $scriptvalue !~ /0/ && $scriptvalue =~ /convert/i;
+    $convertwebbugs = 1 if $webbugvalue !~ /0/ && $webbugvalue =~ /convert/i;
     $stripdangerous = MailScanner::Config::Value('stripdangeroustags',$message);
 
-    #print STDERR "Formvalue = $formvalue\n";
+    #print STDERR "WebBugvalue = $webbugvalue\n";
     #print STDERR "Allowforms = $allowforms and convertforms = $convertforms\n";
 
     # Shortcut the check completely if they want to allow everything
@@ -151,25 +176,15 @@ sub ScanBatch {
           !$stripdangerous) &&
         FindHTMLExploits($message, $id, $ent,
                          $allowiframes, $allowobjects, $allowforms,
-                         $allowscripts,
+                         $allowscripts, $allowwebbugs,
                          $convertiframes, $convertobjects, $convertforms,
-                         $convertscripts,
+                         $convertscripts, $convertwebbugs,
                          $stripdangerous)) {
       $counter++;
       MailScanner::Log::WarnLog('Content Checks: Detected HTML-' .
                                 'specific exploits in %s', $id);
     }
 
-    # Search for message/external-body messages. This is entity-based
-    # as, almost by definition, we won't have the filename worked out
-    # for the file that is external.
-    if ((! MailScanner::Config::Value('allowexternal', $message)) &&
-        FindExternalBody($message, $ent)) {
-      $counter++;
-      MailScanner::Log::WarnLog('Content Checks: Detected and rejected ' .
-                                'external message body in %s', $id);
-    }
-
     # Look for encrypted messages. They can allow some and block some
     # so we need to find the messages then apply the rules to the result.
     my $reason;
@@ -257,11 +272,12 @@ sub CheckAttachmentSizes {
   my($message, $id) = @_;
 
   my($BaseDir, $basefh, $safename, $maxsize, $attachsize, $tnefname);
-  my($unsafename, $counter);
+  my($unsafename, $counter, $minsize);
 
   # Read the configuration setting, value<=0 implies setting not in use.
   $maxsize = MailScanner::Config::Value('maxattachmentsize', $message);
-  return 0 unless $maxsize>=0;
+  $minsize = MailScanner::Config::Value('minattachmentsize', $message);
+  return 0 if $maxsize<0 && $minsize<0;
   $tnefname = $message->{entity2file}{$message->{tnefentity}};
 
   # Get into the directory containing all the attachments
@@ -278,16 +294,31 @@ sub CheckAttachmentSizes {
 
     # "Safe" attachment filename is in $safename, this is what we stat
     $attachsize = -s "$BaseDir/$safename";
-    if ($attachsize > $maxsize) {
+    #print STDERR "\nSafename = $safename\n";
+    #print STDERR "Attachsize=$attachsize\nMin=$minsize\nMax=$maxsize\n";
+    if ($maxsize>=0 && $attachsize > $maxsize &&
+        $safename !~ /^msg[-\d]+\.(txt|html)$/) {
       $unsafename = $message->{safefile2file}{$safename} || $tnefname;
-      MailScanner::Log::InfoLog("Attachment size check: %s > %s (%s)",
-                                $attachsize, $maxsize, $unsafename);
+      MailScanner::Log::InfoLog("Attachment size check: %s > %s (%s) in %s",
+                                $attachsize, $maxsize, $unsafename, $id);
       $message->{otherreports}{$safename} .=
        MailScanner::Config::LanguageValue($message,'attachmenttoolarge') . "\n";
       $message->{othertypes}{$safename}   .= "e";
       $counter++;
       $message->{otherinfected}++;
     }
+    if ($minsize>=0 && $attachsize < $minsize &&
+        $safename !~ /^msg[-\d]+\.(txt|html)$/) {
+      #print STDERR "Attachment is too small\n";
+      $unsafename = $message->{safefile2file}{$safename} || $tnefname;
+      MailScanner::Log::InfoLog("Attachment size check: %s < %s (%s) in %s",
+                                $attachsize, $minsize, $unsafename, $id);
+      $message->{otherreports}{$safename} .=
+       MailScanner::Config::LanguageValue($message,'attachmenttoosmall') . "\n";
+      $message->{othertypes}{$safename}   .= "e";
+      $counter++;
+      $message->{otherinfected}++;
+    }
   }
 
   return $counter;
@@ -354,8 +385,9 @@ sub LastEntity {
 # that contain Microsoft-specific exploits.
 sub FindHTMLExploits {
   my($message, $id, $entity, $allowiframes, $allowobjects, $allowforms,
-     $allowscripts,
+     $allowscripts, $allowwebbugs,
      $convertiframes, $convertobjects, $convertforms, $convertscripts,
+     $convertwebbugs,
      $stripdangerous) = @_;
 
   # Track number of dangerous things found
@@ -370,12 +402,14 @@ sub FindHTMLExploits {
   #$disposition = 'inline' unless $disposition;
   if ($type && $type =~ /text\/html/i &&
       #$disposition !~ /attachment/i &&
-      defined($entity->body) && defined($entity->bodyhandle->path)) {
+      defined($entity->bodyhandle) &&
+      defined($entity->body) &&
+      defined($entity->bodyhandle->path)) {
     $counter += SearchHTMLBody($message, $id, $entity->bodyhandle->path,
                                $allowiframes, $allowobjects, $allowforms,
-                               $allowscripts,
+                               $allowscripts, $allowwebbugs,
                                $convertiframes, $convertobjects, $convertforms,
-                               $convertscripts,
+                               $convertscripts, $convertwebbugs,
                                $stripdangerous);
   }
 
@@ -385,9 +419,9 @@ sub FindHTMLExploits {
   foreach $part (@parts) {
     $counter += FindHTMLExploits($message, $id, $part,
                                $allowiframes, $allowobjects, $allowforms,
-                               $allowscripts,
+                               $allowscripts, $allowwebbugs,
                                $convertiframes, $convertobjects, $convertforms,
-                               $convertscripts,
+                               $convertscripts, $convertwebbugs,
                                $stripdangerous);
   }
 
@@ -399,10 +433,16 @@ sub FindHTMLExploits {
 # that either uses the <IFRAME> tag or the <OBJECT CODEBASE=...> tag.
 sub SearchHTMLBody {
   my($message, $id, $filename, $allowiframes, $allowobjects, $allowforms,
-     $allowscripts,
+     $allowscripts, $allowwebbugs,
      $convertiframes, $convertobjects, $convertforms, $convertscripts,
+     $convertwebbugs,
      $stripdangerous) = @_;
 
+  # We must say we have tags to convert if we are detecting phishing so
+  # that the message is rebuilt properly before delivery.
+  $message->{tagstoconvert} .= 'phishing '
+    if MailScanner::Config::Value('findphishing', $message);
+
   my($fh, $counter, $silentviruses);
   $counter = 0;
   $fh = new FileHandle;
@@ -413,6 +453,7 @@ sub SearchHTMLBody {
     my $iframefound = 0;
     my $formfound   = 0;
     my $scriptfound = 0;
+    my $webbugfound = 0;
     my $codebasefound = 0;
     my $attach = $filename;
     $attach = $1 if $filename =~ /([^\/]+)$/; # Strip off the path
@@ -429,6 +470,9 @@ sub SearchHTMLBody {
       # Find the script tag start
       $scriptfound = 1 if /\<script/i;
 
+      # Find the img tag start
+      $webbugfound = 1 if /\<img/i;
+
       # Find the object tag start
       $inobject = 1 if /\<object/i;
 
@@ -517,6 +561,40 @@ sub SearchHTMLBody {
         $counter++;
       }
     }
+    if ($webbugfound) {
+      ## Log the <img>
+      #MailScanner::Log::InfoLog("HTML Img tag found in message %s from %s",
+      #  $id, $message->{from});
+      #  if MailScanner::Config::Value('logwebbugs', $message);
+      # Mark the message
+      if ($allowwebbugs) {
+        #print STDERR "Web Bug allowed\n";
+        if ($stripdangerous) {
+          #print STDERR "Web Bug stripped\n";
+          $message->{needsstripping} = 1;
+          $message->{bodymodified}   = 1; # Mark it for rebuilding
+          $counter++;
+        }
+      } elsif ($convertwebbugs) {
+        #print STDERR "Web Bug converted\n";
+        $message->{tagstoconvert} .= 'webbug ';
+        $message->{bodymodified}   = 1;
+        #print STDERR "Going to disarm web bugs\n";
+      } else {
+        #print STDERR "Web Bug ignored\n";
+        #
+        # Don't mark potential web bugs as infections, they are just images
+        # until we analyse them in disinfection.
+        #
+        #$message->{otherreports}{"$attach"} .=
+        #  MailScanner::Config::LanguageValue($message, 'foundwebbug') . "\n";
+        #$message->{othertypes}{"$attach"}   .= "c";
+        #$message->{otherinfected}++;
+        #$message->{silent} = 1 if $silentviruses =~ / HTML-WebBug /i;
+        #$counter++;
+        1;
+      }
+    }
     if ($codebasefound) {
       if ($allowobjects) {
         if ($stripdangerous) {
