@@ -2,7 +2,7 @@
 #   MailScanner - SMTP E-Mail Virus Scanner
 #   Copyright (C) 2002  Julian Field
 #
-#   $Id: SweepViruses.pm,v 1.49.2.71 2004/05/01 11:38:01 jkf Exp $
+#   $Id: SweepViruses.pm,v 1.49.2.92 2004/12/29 17:27:34 jkf Exp $
 #
 #   This program is free software; you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
@@ -38,10 +38,10 @@ no  strict 'subs'; # Allow bare words for parameter %'s
 use POSIX qw(:signal_h setsid); # For Solaris 9 SIG bug workaround
 use DirHandle;
 
-use vars qw($VERSION);
+use vars qw($VERSION $ScannerPID);
 
 ### The package version, both in 1.23 style *and* usable by MakeMaker:
-$VERSION = substr q$Revision: 1.49.2.71 $, 10;
+$VERSION = substr q$Revision: 1.49.2.92 $, 10;
 
 # Locking definitions for flock() which is used to lock the Lock file
 my($LOCK_SH) = 1;
@@ -59,6 +59,8 @@ $SAVIinuse       = 0;
 my($Clam, $Claminuse, %Clamwatchfiles);
 $Claminuse       = 0;
 %Clamwatchfiles  = ();
+# So we can kill virus scanners when we are HUPped
+$ScannerPID = 0;
 
 
 
@@ -74,6 +76,17 @@ my (
    ) = (0,1,2,3,4);
 
 my %Scanners = (
+  generic => {
+    Name		=> 'Generic',
+    Lock		=> 'GenericBusy.lock',
+    CommonOptions	=> '',
+    DisinfectOptions	=> '-disinfect',
+    ScanOptions		=> '',
+    InitParser		=> \&InitGenericParser,
+    ProcessOutput	=> \&ProcessGenericOutput,
+    SupportScanning	=> $S_SUPPORTED,
+    SupportDisinfect	=> $S_NONE,
+  },
   sophossavi => {
     Name		=> 'SophosSAVI',
     Lock		=> 'SophosBusy.lock',
@@ -224,10 +237,10 @@ my %Scanners = (
   "nod32-1.99"		=> {
     Name		=> 'Nod32',
     Lock		=> 'Nod32Busy.lock',
-    CommonOptions	=> '--arch --all --log-brief',
+    CommonOptions	=> '--arch --all',
     DisinfectOptions	=> '--action clean --action-uncl none',
     ScanOptions		=> '',
-    InitParser		=> \&InitNOD32Parser,
+    InitParser		=> \&InitNOD32199Parser,
     ProcessOutput	=> \&ProcessNOD32Output,
     SupportScanning	=> $S_SUPPORTED,
     SupportDisinfect	=> $S_SUPPORTED,
@@ -306,8 +319,8 @@ my %Scanners = (
     ScanOptions         => '',
     InitParser          => \&InitBitdefenderParser,
     ProcessOutput       => \&ProcessBitdefenderOutput,
-    SupportScanning     => $S_BETA,
-    SupportDisinfect    => $S_BETA,
+    SupportScanning     => $S_SUPPORTED,
+    SupportDisinfect    => $S_SUPPORTED,
   },
   "drweb"   => {
     Name		=> 'DrWeb',
@@ -328,8 +341,8 @@ my %Scanners = (
     ScanOptions         => '',
     InitParser          => \&InitNormanParser,
     ProcessOutput       => \&ProcessNormanOutput,
-    SupportScanning     => $S_BETA,
-    SupportDisinfect    => $S_BETA,
+    SupportScanning     => $S_SUPPORTED,
+    SupportDisinfect    => $S_SUPPORTED,
   },
   "css" => {
     Name                => 'SYMCScan',
@@ -351,8 +364,30 @@ my %Scanners = (
     InitParser          => \&InitAvgParser,
     ProcessOutput       => \&ProcessAvgOutput,
     SupportScanning     => $S_SUPPORTED,
+    SupportDisinfect    => $S_NONE,
+  },
+  "vexira"   => {
+    Name                => 'Vexira',
+    Lock                => 'VexiraBusy.lock',
+    CommonOptions       => '--allfiles -s -z -noboot -nombr -r1 -rs -lang=EN --alltypes',
+    DisinfectOptions    => '-e',
+    ScanOptions         => '',
+    InitParser          => \&InitVexiraParser,
+    ProcessOutput       => \&ProcessVexiraOutput,
+    SupportScanning     => $S_SUPPORTED,
     SupportDisinfect    => $S_SUPPORTED,
   },
+  "symscanengine"	=> {
+    Name		=> 'SymantecScanEngine',
+    Lock		=> 'SymScanEngineBusy.lock',
+    CommonOptions	=> '-details -recurse',
+    DisinfectOptions	=> '-mode scanrepair',
+    ScanOptions		=> '-mode scan',
+    InitParser		=> \&InitSymScanEngineParser,
+    ProcessOutput	=> \&ProcessSymScanEngineOutput,
+    SupportScanning	=> $S_BETA,
+    SupportDisinfect	=> $S_BETA,
+  },
   "none"		=> {
     Name		=> 'None',
     Lock		=> 'NoneBusy.lock',
@@ -388,7 +423,14 @@ sub initialise {
 
 sub InitialiseClam {
   # Initialise ClamAV Module
-  require Mail::ClamAV;
+  MailScanner::Log::DieLog("ClamAV Perl module not found, did you install it?")
+    unless eval 'require Mail::ClamAV';
+
+  my $ver = $Mail::ClamAV::VERSION + 0.0;
+  MailScanner::Log::DieLog("ClamAV Perl module must be at least version 0.12" .
+                           " and you only have version %.2f, and ClamAV must" .
+                           " be at least version 0.80", $ver)
+    unless $ver >= 0.12;
 
   $Clam = new Mail::ClamAV(Mail::ClamAV::retdbdir())
     or MailScanner::Log::DieLog("ClamAV Module ERROR:: Could not load " .
@@ -412,12 +454,15 @@ sub InitialiseClam {
   MailScanner::Log::DieLog("None of the files matched by the \"Monitors " .
     "For ClamAV Updates\" patterns exist!") unless $filecount>0;
 
+  MailScanner::Log::WarnLog("\"Allow Password-Protected Archives\" should be set to just yes or no when using clamavmodule virus scanner")
+    unless MailScanner::Config::IsSimpleValue('allowpasszips');
 }
 
 
 sub InitialiseSAVI {
   # Initialise Sophos SAVI library
-  require SAVI;
+  MailScanner::Log::DieLog("SAVI Perl module not found, did you install it?")
+    unless eval 'require SAVI';
 
   my $SAVIidedir = MailScanner::Config::Value('sophoside');
   $SAVIidedir = '/usr/local/Sophos/ide' unless $SAVIidedir;
@@ -670,6 +715,9 @@ sub MergeReports {
       #print STDERR "\tattachment \"$attachment\" has text \"$text\"\n";
       #print STDERR "\tEntity of \"$attachment\" is \"" . $message->{file2entity} . "\"\n";
       next unless $text;
+
+      # Sanitise the reports a bit
+      $text =~ s/\s{20,}/ /g;
       $message->{virusreports}{"$attachment"} .= $text;
     }
     MailScanner::Log::InfoLog("Infected message %s came from %s",
@@ -812,6 +860,7 @@ sub TryOneCommercial {
       # In the parent
       local $SIG{ALRM} = sub { $TimedOut = 1; die "Command Timed Out" };
       alarm MailScanner::Config::Value('virusscannertimeout');
+      $ScannerPID = $pid;
       # Only process the output if we are scanning, not disinfecting
       if ($disinfect) {
         # Tell sweep to disinfect all files
@@ -857,6 +906,7 @@ sub TryOneCommercial {
 
   # Note to self: I only close the KID in the parent, not in the child.
   MailScanner::Log::DebugLog("Completed scanning by $scanner");
+  $ScannerPID = 0; # Not running a scanner any more
 
   # Catch failures other than the alarm
   MailScanner::Log::DieLog("Commercial virus checker failed with real error: $@")
@@ -928,9 +978,36 @@ sub ClamAVModule {
                                   "$dirname/$childname", $!);
 
     # Scan all the files in the subdirectory
+    # check to see if rar is available. If it is we don't want to
+    # have clamav check for password protected since that has already
+    # been done and will be reported correctly
+    # if we are not allowing password protected archives and do not have rar
+    # then have clamav check for password protected archives but it will
+    # be reported as a virus (at least it will block passworded rar files)
+
+    # Do we have an unrar on the path?
+    #my $unrar = `which unrar`;
+    #chomp($unrar);
+    #my $haverar = 0;
+    ##MailScanner::Log::InfoLog("*********Check Rar Results : %s\n",$unrar);
+    #$haverar = 1 if $unrar !~ /no unrar in/i && $unrar ne "";
+
     while($filename = $child->read()) {
       next unless -f "$dirname/$childname/$filename"; # Only check files
-      $results = $Clam->scan("$dirname/$childname/$filename", Mail::ClamAV::CL_ARCHIVE());
+      # JKF Patch suggested by Rick Cooper to support unrar and OLE2 files
+      #$results = $Clam->scan("$dirname/$childname/$filename",
+      #                       Mail::ClamAV::CL_SCAN_ARCHIVE());
+      if (MailScanner::Config::Value('allowpasszips')) { # || $haverar) {
+        $results = $Clam->scan("$dirname/$childname/$filename",
+                               Mail::ClamAV::CL_SCAN_ARCHIVE() |
+                               Mail::ClamAV::CL_SCAN_OLE2());
+      } else {
+        $results = $Clam->scan("$dirname/$childname/$filename",
+                               Mail::ClamAV::CL_SCAN_ARCHIVE() |
+                               Mail::ClamAV::CL_SCAN_BLOCKENCRYPTED() |
+                               Mail::ClamAV::CL_SCAN_OLE2());
+      }
+
       unless ($results) {
         print "ERROR:: $results" . ":: $dirname/$childname/$filename\n";
         next;
@@ -1009,6 +1086,11 @@ sub SophosSAVI {
   $dir->close;
 }
 
+# Initialise any state variables the Generic output parser uses
+sub InitGenericParser {
+  ;
+}
+
 # Initialise any state variables the Sophos SAVI output parser uses
 sub InitSophosSAVIParser {
   ;
@@ -1078,6 +1160,12 @@ sub InitNOD32Parser {
   $NOD32InHeading = 1;
 }
 
+# Initialise any state variables the Nod32 1.99 and above output parser uses
+sub InitNOD32199Parser {
+  $NOD32Version = undef;
+  $NOD32InHeading = 2;
+}
+
 # Initialise any state variables the AntiVir output parser uses
 sub InitAntiVirParser {
   ;
@@ -1137,6 +1225,17 @@ sub InitAvgParser {
   ;
 }
 
+# Initialise any state variables the Vexira output parser uses
+sub InitVexiraParser {
+  ;
+}
+
+# Initialise any state variables the ScanEngine output parser uses
+sub InitSymScanEngineParser {
+  ;
+}
+
+
 # These functions must be called with, in order:
 # * The line of output from the scanner
 # * The MessageBatch object the reports are written to
@@ -1167,6 +1266,7 @@ sub ProcessClamAVModOutput {
 
   chomp $line;
   $logout = $line;
+  $logout =~ s/\s{20,}/ /g;
   #$logout =~ s/%/%%/g;
 
   #print STDERR "Output is \"$logout\"\n";
@@ -1192,6 +1292,29 @@ sub ProcessClamAVModOutput {
   }
 }
 
+sub ProcessGenericOutput {
+  my($line, $infections, $types, $BaseDir, $Name) = @_;
+  my($logout, $keyword, $virusname, $filename);
+  my($id, $part, @rest, $report);
+
+  chomp $line;
+  $logout = $line;
+  $logout =~ s/\s{20,}/ /g;
+  ($keyword, $virusname, $filename) = split(/::/, $line, 3);
+
+  MailScanner::Log::InfoLog("GenericScanner::%s", $logout);
+  return 1 if $keyword =~ /^error/i;
+  return 0 if $keyword =~ /^clean|^info/i;
+
+  # Must be an infection report
+  ($id, $part, @rest) = split(/\//, $filename);
+  $report = $Name . ': ' if $Name;
+  $infections->{"$id"}{"$part"} .= "$report$part was infected by $virusname\n";
+  $types->{"$id"}{"$part"} .= "v"; # it's a real virus
+  return 1;
+}
+
+
 sub ProcessSophosSAVIOutput {
   my($line, $infections, $types, $BaseDir, $Name) = @_;
   my($logout, $keyword, $virusname, $filename);
@@ -1199,6 +1322,7 @@ sub ProcessSophosSAVIOutput {
 
   chomp $line;
   $logout = $line;
+  $logout =~ s/\s{20,}/ /g;
   #$logout =~ s/%/%%/g;
 
   ($keyword, $virusname, $filename) = split(/:: /, $line, 3);
@@ -1232,6 +1356,7 @@ sub ProcessSophosOutput {
   chomp $line;
   $logout = $line;
   $logout =~ s/%/%%/g;
+  $logout =~ s/\s{20,}/ /g;
   MailScanner::Log::InfoLog($logout) if $line =~ /error/i;
   # JKF Improved to handle multi-part split archives,
   # JKF which Sophos whinges about
@@ -1308,6 +1433,7 @@ sub ProcessMcAfeeOutput {
   $report = "$lastline$currentline";
   $logout = $report;
   $logout =~ s/%/%%/g;
+  $logout =~ s/\s{20,}/ /g;
   MailScanner::Log::InfoLog($logout);
   # note: '$dot' does not become '.'
   ($dot, $id, $part, @rest) = split(/\//, $lastline);
@@ -1332,6 +1458,7 @@ sub ProcessCommandOutput {
   chomp $line;
   $logout = $line;
   $logout =~ s/%/%%/g;
+  $logout =~ s/\s{20,}/ /g;
   MailScanner::Log::InfoLog($logout) if $line =~ /error/i;
   if ($line =~ /(is|could be) a (security risk|virus construction|joke program)/) {
     # Reparse the rest of the line to turn it into an infection report
@@ -1367,6 +1494,7 @@ sub ProcessInoculateOutput {
   chomp $line;
   $logout = $line;
   $logout =~ s/%/%%/g;
+  $logout =~ s/\s{20,}/ /g;
   MailScanner::Log::InfoLog($logout) if $line =~ /error/i;
   #JKF MailScanner::Log::WarnLog($line) if $line =~ /Error/i;
   return 0 unless $line =~ /is infected by virus:/i;
@@ -1442,6 +1570,7 @@ sub ProcessInoculanOutput {
   chomp $line;
   $logout = $line;
   $logout =~ s/%/%%/g;
+  $logout =~ s/\s{20,}/ /g;
   MailScanner::Log::InfoLog($logout) if $line =~ /error/i;
   #JKF MailScanner::Log::WarnLog($line) if $line =~ /Error/i;
   return 0 unless $line =~ /was infected by virus/i;
@@ -1482,6 +1611,7 @@ sub ProcessKaspersky_4_5Output {
   $line =~ s/^\[[^\]]+\] //;
   $logout = $line;
   $logout =~ s/%/%%/g;
+  $logout =~ s/\s{20,}/ /g;
   MailScanner::Log::InfoLog($logout);
 
   # Sample outputs for an unpacked and a packed virus
@@ -1538,6 +1668,7 @@ sub ProcessKasperskyOutput {
       $report = $line;
       $logout = $line;
       $logout =~ s/%/%%/g;
+      $logout =~ s/\s{20,}/ /g;
       MailScanner::Log::InfoLog($logout);
       $line =~ s/^$BaseDir//;
       $line =~ s/(.*) infected:.*/\.$1/; # To handle long paths again
@@ -1580,6 +1711,7 @@ sub ProcessKavDaemonClientOutput {
     $report = $line;
     $logout = $line;
     $logout =~ s/%/%%/g;
+    $logout =~ s/\s{20,}/ /g;
     MailScanner::Log::InfoLog($logout);
     $line =~ s/^$BaseDir//;
     $line =~ s/(.*)\sinfected:.*/\.$1/; # To handle long paths again
@@ -1610,7 +1742,8 @@ sub ProcessFSecureOutput {
   if ($fsecure_InHeader < 0 && $line =~ /version ([\d.]+)/i &&
       !$fsecure_Version) {
     $fsecure_Version = $1 + 0.0;
-    $fsecure_InHeader -= 2 if $fsecure_Version >= 4.51;
+    $fsecure_InHeader -= 2 if $fsecure_Version >= 4.51 &&
+                              $fsecure_Version < 4.60;
     #MailScanner::Log::InfoLog("Found F-Secure version $1=$fsecure_Version\n");
     return 0;
   }
@@ -1626,6 +1759,7 @@ sub ProcessFSecureOutput {
   $report = $line;
   $logout = $line;
   $logout =~ s/%/%%/g;
+  $logout =~ s/\s{20,}/ /g;
   MailScanner::Log::InfoLog($logout);
 
   # If we are running the new version then there's a totally new parser here
@@ -1634,6 +1768,17 @@ sub ProcessFSecureOutput {
     #./g4UFLJR23090/Keld Jrn Simonsen: Infected: EICAR_Test_File [F-Prot]
     #./g4UFLJR23090/Keld Jrn Simonsen: Infected: EICAR-Test-File [AVP]
     #./g4UFLJR23090/cokegift.exe: Infected:   is a joke program [F-Prot]
+    # Version 4.61:
+    #./eicar.com: Infected: EICAR_Test_File [Libra]
+    #./eicar.com: Infected: EICAR Test File [Orion]
+    #./eicar.com: Infected: EICAR-Test-File [AVP]
+    #./eicar.doc: Infected: EICAR_Test_File [Libra]
+    #./eicar.doc: Infected: EICAR Test File [Orion]
+    #./eicar.doc: Infected: EICAR-Test-File [AVP]
+    #[./eicar.zip] eicar.com: Infected: EICAR_Test_File [Libra]
+    #[./eicar.zip] eicar.com: Infected: EICAR Test File [Orion]
+    #[./eicar.zip] eicar.com: Infected: EICAR-Test-File [AVP]
+
 
     return 0 unless $line =~ /: Infected: /;
     # The last 3 words are "Infected:" + name of virus + name of scanner
@@ -1719,6 +1864,7 @@ sub ProcessFProtOutput {
   $report = $line;
   $logout = $line;
   $logout =~ s/%/%%/g;
+  $logout =~ s/\s{20,}/ /g;
   MailScanner::Log::InfoLog($logout);
   if ($line =~ /(is|could be) a (security risk|virus construction)/) {
     $line =~ s/(is|could be) a (security risk|virus construction).*$/Infection: /;
@@ -1798,15 +1944,32 @@ sub ProcessNOD32Output {
   chomp $line;
   $logout = $line;
   $logout =~ s/%/%%/g;
+  $logout =~ s/\s{20,}/ /g;
   MailScanner::Log::WarnLog($logout) if $line =~ /error/i;
 
-  if (!$NOD32Version && $NOD32InHeading && /^NOD32.*Version.*([\d.]+)/) {
+  # Yet another new NOD32 parser! :-(
+  # This one is for 2.04 in which the output, again, looks totally different
+  # to all the previous versions.
+  if ($line =~
+      /^object=\"file\",\s*name=\"([^\"]+)\",\s*(virus=\"([^\"]+)\")?/i) {
+    my($fileentry, $virusname) = ($1,$3);
+    $fileentry =~ s/^$BaseDir//;
+    ($dot, $id, $part, @rest) = split(/\//, $fileentry);
+    $part =~ s/^.*\-\> //g;
+    $report = "Found virus $virusname in $part";
+    $report = $Name . ': '. $report if $Name;
+    $infections->{"$id"}{"$part"} .= $report . "\n";
+    $types->{"$id"}{"$part"} .= "v"; # it's a real virus
+    return 1;
+  }
+
+
+  if (!$NOD32Version && $NOD32InHeading>0 && $line =~ /^NOD32.*Version[^\d]*([\d.]+)/) {
     $NOD32Version = $1;
-    $NOD32InHeading = 0;
+    $NOD32InHeading--; # = 0;
     return 0;
   }
-  $NOD32InHeading = 0 if /^$/;
-
+  $NOD32InHeading-- if /^$/; # Was = 0
   return 0 unless $line =~ /\s-\s/i;
 
   if ($NOD32Version >= 1.990) {
@@ -1833,6 +1996,7 @@ sub ProcessNOD32Output {
     #$line="./".$part4;
     $logout = $line;
     $logout =~ s/%/%%/g;
+    $logout =~ s/\s{20,}/ /g;
     MailScanner::Log::InfoLog($logout);
     $report = $line;
     $infected = $line;
@@ -1892,6 +2056,7 @@ sub ProcessAntiVirOutput {
   if ($line =~ /.*!Virus!.*/) {
     $logout = $line;
     $logout =~ s/%/%%/g;
+    $logout =~ s/\s{20,}/ /g;
     MailScanner::Log::InfoLog($logout);
     ($dot,$id,$part,@rest) = split(/\//, $line);
     # The Filename is all, except the last two comma seperated elements
@@ -1909,6 +2074,7 @@ sub ProcessAntiVirOutput {
   if ($line =~ /^ALERT:/) {
     $logout = $line;
     $logout =~ s/%/%%/g;
+    $logout =~ s/\s{20,}/ /g;
     MailScanner::Log::InfoLog($logout);
     # Get rid of the virus name
     $line =~ s/^ALERT: \[[^\]]+\] //;
@@ -1949,6 +2115,7 @@ sub ProcessPandaOutput {
 
   $logout = $line;
   $logout =~ s/%/%%/g;
+  $logout =~ s/\s{20,}/ /g;
   MailScanner::Log::InfoLog($logout);
 
   # EXAMPLE OUTPUT PLEASE? -- nwp 6/5/02
@@ -2046,6 +2213,7 @@ sub ProcessRavOutput {
   if ($line =~ /\t+(Infected|Suspicious): /i) {
     $logout = $line;
     $logout =~ s/%/%%/g;
+    $logout =~ s/\s{20,}/ /g;
     MailScanner::Log::InfoLog($logout);
     # Get to relevant filename in a reasonably but not
     # totally robust manner (*impossible* to be totally robust
@@ -2078,7 +2246,9 @@ sub ProcessDrwebOutput {
   return 0 unless $line =~ /^(.+)\s+infected\s+with\s+(.*)$/i;
 
   my ($file, $virus) = ($1, $2);
-  MailScanner::Log::InfoLog($line);
+  my $logout = $line;
+  $logout =~ s/\s{20,}/ /g;
+  MailScanner::Log::InfoLog("%s", $logout);
 
   # Sample output:
   #
@@ -2119,6 +2289,7 @@ sub ProcessClamAVOutput {
     chomp $line;
     $logline = $line;
     $logline =~ s/%/%%/g;
+    $logline =~ s/\s{20,}/ /g;
     MailScanner::Log::WarnLog($logline);
     return 0;
   }
@@ -2141,7 +2312,7 @@ sub ProcessClamAVOutput {
     MailScanner::Log::InfoLog("ProcessClamAVOutput: %s", $1);
     return 0;
   }
-  return 0 if /^  /;  # "  inflating", "  deflating.." from --unzip
+  return 0 if /^  |^Extracting|module failure$/;  # "  inflating", "  deflating.." from --unzip
   if ($clamav_archive && /^$clamav_archive:/)
   {
     $clamav_archive = "";
@@ -2150,7 +2321,25 @@ sub ProcessClamAVOutput {
 
   return 0 if /OK$/; 
   
-  MailScanner::Log::InfoLog("%s", $line);
+  $logline = $line;
+  $logline =~ s/\s{20,}/ /g;
+  MailScanner::Log::InfoLog("%s", $logline);
+
+  #(Real infected archive: /var/spool/MailScanner/incoming/19746/./i75EFmSZ014248/eicar.rar)
+  if (/^\(Real infected archive: (.*)\)$/)
+  {
+     my ($file, $ReportStart);
+     $file = $1;
+     $file =~ s/^(.\/)?$BaseDir\/?//;
+     $file =~ s/^\.\///;
+     my ($id,$part) = split /\//, $file, 2;
+
+     $ReportStart = $part;
+     $ReportStart = $Name . ': ' . $ReportStart if $Name;
+     $infections->{"$id"}{"$part"} .= "$ReportStart contains a virus\n";
+     $types->{"$id"}{"$part"} .= "v";
+     return 1;
+  }
 
   if (/^(\(raw\) )?(.*?): (.*) FOUND$/)
   {
@@ -2167,6 +2356,10 @@ sub ProcessClamAVOutput {
 	 $file = $2;
      }     
      
+     ## If it doesn't start with $BaseDir/./ then it isn't a real report
+     # Don't release this just yet
+     #return 0 unless $file =~ /^\/$BaseDir\/\.\//;
+
      $file =~ s/^(.\/)?$BaseDir\/?//;
      $file =~ s/^\.\///;
      my ($id,$part) = split /\//, $file, 2;
@@ -2177,6 +2370,10 @@ sub ProcessClamAVOutput {
     $types->{"$id"}{"$part"} .= "v";
     return 1;
   }
+  ## If it doesn't start with $BaseDir/./ then it isn't a real report
+  # Don't release this just yet
+  #return 0 unless /^\/$BaseDir\/\.\//;
+
   if (/^(.*?): File size limit exceeded\.$/) {
     return 0;
   #  my ($file, $ReportStart);
@@ -2242,7 +2439,7 @@ sub ProcessTrendOutput {
     $infections->{$id}{$part} .= $Name . ': ' if $Name;
     $infections->{$id}{$part} .= "Found virus $virus in file $trend_prevline\n";
     $types->{$id}{$part}      .= "v"; # so we know what to tell sender
-    MailScanner::Log::InfoLog("Trend found %s in $trend_prevline", $virus);
+    MailScanner::Log::InfoLog("Trend found %s in %s", $virus, $trend_prevline);
     return 1;
    }
   return 0;
@@ -2258,8 +2455,10 @@ sub ProcessBitdefenderOutput {
   return 0 unless $line =~ /\tinfected: ([^\t]+)$/;
 
   my $virus = $1;
+  my $logout = $line;
+  $logout =~ s/\s{20,}/ /g;
   #print STDERR "virus = \"$virus\"\n";
-  MailScanner::Log::InfoLog("%s", $line);
+  MailScanner::Log::InfoLog("%s", $logout);
 
   # Sample output:
   #
@@ -2305,8 +2504,10 @@ sub ProcessNormanOutput {
   return 0 unless $line =~ /^[^']+'([^']+)' -> '([^']+)'\s*$/;
   my ($filename, $virus) = ($1, $2);
 
-  print STDERR "virus = \"$virus\"\n";
-  MailScanner::Log::InfoLog("%s", $line);
+  #print STDERR "virus = \"$virus\"\n";
+  my $logout = $line;
+  $logout =~ s/\s{20,}/ /g;
+  MailScanner::Log::InfoLog("%s", $logout);
 
   # Remove $BaseDir from front of filename if it's there
   $filename =~ s/^$BaseDir\///;
@@ -2317,7 +2518,7 @@ sub ProcessNormanOutput {
 
   $part =~ s/ : .*$//; # Remove archive member filename
 
-  print STDERR "id = $id\npart = $part\n";
+  #print STDERR "id = $id\npart = $part\n";
   $infections->{$id}{$part} .= $Name . ': ' if $Name;
   $infections->{$id}{$part} .= "Found virus $virus in file $part\n";
   $types->{$id}{$part}      .= "v"; # so we know what to tell sender
@@ -2337,6 +2538,7 @@ sub ProcessCSSOutput {
   {
     $logline = $line;
     $logline =~ s/%/%%/g;
+    $logline =~ s/\s{20,}/ /g;
     MailScanner::Log::WarnLog($logline);
     return 0;
   }
@@ -2404,13 +2606,18 @@ sub ProcessAvgOutput {
   #./1B978O-0000g2-Iq/eicar.com  Virus identified  EICAR_Test (+2)
   #./1B978O-0000g2-Iq/eicar.zip:\eicar.com  Virus identified  EICAR_Test (+2)
 
+  # Remove all the duff carriage-returns from the line
+  $line =~ s/[\r\n]//g;
+
   #print STDERR "Line: $line\n";
   return 0 unless $line =~ /Virus identified  (.+)$/;
 
   my $virus = $1;
   #print STDERR "Line: $line\n";
   #print STDERR "virus = \"$virus\"\n";
-  MailScanner::Log::InfoLog("%s", $line);
+  my $logout = $line;
+  $logout =~ s/\s{20,}/ /g;
+  MailScanner::Log::InfoLog("%s", $logout);
 
   # Change all the spaces into / for the split coming up
   # Also the second variant prepends the archive name to the
@@ -2433,6 +2640,83 @@ sub ProcessAvgOutput {
   return 1;
 }
 
+sub ProcessVexiraOutput {
+  my($line, $infections, $types, $BaseDir, $Name) = @_;
+  chomp $line;
+  # Sample output:
+  # ALERT: [Eicar-Test-Signatur virus] ./gBJNiNQG014777/eicar.zip --> eicar.com <<< Contains code of the Eicar-Test-Signatur virus
+  # ALERT: [Eicar-Test-Signatur virus] ./gBJNiNQG014777/eicar.com <<< Contains code of the Eicar-Test-Signatur virus
+  # ALERT: [Eicar-Test-Signatur virus] ./gBJNiNQG014777/eicar.doc <<< Contains code of the Eicar-Test-Signatur virus
+
+  print STDERR "Line: $line\n";
+  return 0 unless $line =~ /^ALERT: \[([^\]]+)\] /;
+
+  my $virus = $1;
+  print STDERR "Line = $line\n";
+  print STDERR "virus = \"$virus\"\n";
+  my $logout = $line;
+  $logout =~ s/\s{20,}/ /g;
+  MailScanner::Log::InfoLog("%s", $logout);
+
+  # Change all the spaces into / for the split coming up
+  # Also the second variant prepends the archive name to the
+  # infected filename with a:\ so we need to change that to
+  # something else. I chose another / so it would end up in the
+  # @rest wich is also why I changed the \s+ to /
+  # then Remove path elements before /./ leaving just id/part/rest
+
+  #$line =~ s/^ALERT: //g; Patch provided by Alex Kerkhove
+  $line =~ s/^ALERT: //;
+  $line =~ s/\[.+\] *//g;
+  $line =~ s/\.\///;
+  #$line =~ s/^([^\/]+)\///g; Patch provided by Alex Kerkhove
+  $line =~ s/^([^\/]+)\///;
+  my $id = $1;
+  $line =~ /^(.+) <<< (.+)$/;
+  my $part = $1;
+  $part =~ s/ -->.*$//g;
+  print STDERR "id:$id:part = $part\n";
+  print STDERR "$Name : Found virus $virus in file $part ID:$id\n";
+  $infections->{$id}{$part} .= $Name . ': ' if $Name;
+  $infections->{$id}{$part} .= "Found $virus in file $part\n";
+  $types->{$id}{$part}      .= "v"; # so we know what to tell sender
+  return 1;
+}
+
+sub ProcessSymScanEngineOutput {
+  my($line, $infections, $types, $BaseDir, $Name) = @_;
+  my($logout, $virusid, $virusname, $filename, $action, $shortname);
+  my($file, @files, $virus_found);
+  my($dot, $id, $part, @rest, $report);
+
+  chomp $line;
+
+  # Split all lines that start with a '.'
+  @files=split(/\|\./, $line);
+  $virus_found=0;
+  foreach $file (@files) {
+	$file=~/^(.*) had [0-9]+ infection\(s\):\|File Name:\s+([^\|]*)\|Virus Name:\s+([^\|]*)\|Virus ID:\s+([^\|]*)\|(.*)$/ || next;
+	$filename=".$1";	# We removed the '.', so put it back
+	$shortname=$2;
+	$virusname=$3;
+	$virusid=$4;
+	$action=$5;
+	# Ignore it if it's not a real virus
+	if(int($virusid) < 0) {
+		next;
+	}
+	($dot, $id, $part, @rest) = split(/\//, $filename);
+	MailScanner::Log::InfoLog("SymantecScanEngine::$shortname $virusname ");
+	$report = $Name . ': ' if $Name;
+	$infections->{"$id"}{"$part"}
+	.= "$report$part was infected: $virusname\n";
+	$types->{"$id"}{"$part"} .= "v"; # it's a real virus
+	$virus_found=1;
+  }
+  return $virus_found;
+}
+
+
 
 # Should be called when we're about to try to run some code to
 # scan or disinfect (after checking that code is present).
