@@ -2,7 +2,7 @@
 #   MailScanner - SMTP E-Mail Virus Scanner
 #   Copyright (C) 2002  Julian Field
 #
-#   $Id: ZMailer.pm,v 1.1.2.23 2004/03/31 19:05:27 jkf Exp $
+#   $Id: ZMailer.pm,v 1.1.2.33 2004/12/18 12:25:47 jkf Exp $
 #
 #   This program is free software; you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
@@ -41,7 +41,7 @@ use DirHandle;
 use vars qw($VERSION);
 
 ### The package version, both in 1.23 style *and* usable by MakeMaker:
-$VERSION = substr q$Revision: 1.1.2.23 $, 10;
+$VERSION = substr q$Revision: 1.1.2.33 $, 10;
 
 # Command-line options you need to give to sendmail to sensibly process a
 # message that is piped to it. Still need to add "-f" for specifying the
@@ -256,7 +256,7 @@ sub ReadQf {
       next;
     }
 
-    last if( $InHeader && /^\s*$/ ); # One blank line ends whith the header part
+    last if( $InHeader && /^$/ ); # One empty line ends whith the header part
 
     $Line = $_;
     unless( $InHeader ) {
@@ -276,7 +276,7 @@ sub ReadQf {
         $message->{from} = lc($from);
         $FROMFound = 1; # We have found the sender
       }
-      if ($Line =~ /^channel error/) {
+      if ($Line =~ /^channel error/i) {
         $from = "";
         $message->{from} = lc($from);
         $FROMFound = 1; # We have found the (NULL) sender
@@ -286,7 +286,7 @@ sub ReadQf {
         # PERT-BBY
         # handle envelope rcvdfrom to generate fake Received: for SA
         $RcvdFromLine = $Line;
-        $RcvdFromLine =~ s/^rcvdfrom //;
+        $RcvdFromLine =~ s/^rcvdfrom //i;
         $message->{zmenvrcvdfrom} = $RcvdFromLine;
         if ($ip =~ /\[(\d+\.\d+\.\d+.\d+)\]/) {
           $message->{clientip} = $1;
@@ -301,11 +301,24 @@ sub ReadQf {
         # handle envelope with to generate fake Received: for SA
         $message->{zmenvwith} = $1;
       }
+      if ($Line =~ /^rcvdfrom .*\ssmtp-auth:\s"([^"]+)"/i) {
+        $message->{authenticateduser} = $1;
+        $message->{authenticateduser} =~ s/\s//g;
+        unless( $message->{authenticateduser} =~ /@/ ) {
+          my $domainauth= MailScanner::Config::Value('domainauth');
+          if( $domainauth ne "no" ) {
+            $message->{authenticateduser} .= "\@$domainauth";
+          } else {
+            MailScanner::Log::WarnLog("Batch: There are authenticated users " .
+	             "without domain but 'Default Domain Without Auth' is not defined" );
+            undef $message->{authenticateduser};
+          }
+        }
+      }
       push @{$message->{metadata}}, $Line; # Put non-headers into @metadata
-    }
-    else { # Inside Headers
+    } else { # Inside Headers
       push @{$message->{headers}}, $Line;
-      if ($Line =~ /^Subject:\s+(\S.*)$/i) {
+      if ($Line =~ /^Subject:\s*(\S.*)?$/i) {
         $message->{subject} = $1;
       }
     }
@@ -454,7 +467,7 @@ sub PrependHeader {
   }
 
   $message->{metadata}[$oldlocation] =~
-      s/^$key\s+/$key $newvalue$sep/i;
+      s/^$key\s*/$key $newvalue$sep/i;
 }
 
 # REVISO LEOH
@@ -531,10 +544,20 @@ sub TextEndsHeader {
 sub AddRecipients {
   my $this = shift;
   my($message, @recips) = @_;
-  my($recip);
-  foreach $recip (@recips) {
-    push @{$message->{metadata}}, "to <$recip>";
+  #my($recip);
+  #foreach $recip (@recips) {
+  #  push @{$message->{metadata}}, "to <$recip>";
+  #PERT-BBY add the envelope format to addresses:
+  for (@recips) { s/^(.*)$/to <$1>/ };
+  my ($line);
+  my $pos=0;
+  #PERT-BBY now, look for first 'to '
+  foreach $line (@{$message->{metadata}}) {
+    last if ($line =~ /^to /);
+    $pos++;
   }
+  #PERT-BBY and insert the new ones here
+  splice(@{$message->{metadata}} , $pos, 0, @recips);
 }
 
 # Delete the original recipients from the message. We'll add some
@@ -542,12 +565,18 @@ sub AddRecipients {
 #REVISO LEOH
 sub DeleteRecipients {
   my $this = shift;
-  my($message) = @_;
+  my($message, $user) = @_;
 
+  $user= quotemeta($user) if( $user );
+  #my $userre=$user ? qr/.*\b$user\b/ : qr //;
+  #my $userre=$user ?  qr/(:?.*RCPT=rfc822;|\s*<?)$user(:?\s+.*|>?\s*)/i : qr /.*/;
+  my $userre=$user ?  qr/(?:.*RCPT=rfc822;$user\s|\s*<?$user(?:[>\s]|$))/i : qr //;
   my($linenum);
   for ($linenum=0; $linenum<@{$message->{metadata}}; $linenum++) {
     # Looking for "recipient" lines
-    next unless $message->{metadata}[$linenum] =~ /to /;
+    #next unless $message->{metadata}[$linenum] =~ /^to(:?dsn)?\s$userre/;
+    #next unless $message->{metadata}[$linenum] =~ /^to(:?dsn)?\s$userre/i;
+    next unless $message->{metadata}[$linenum] =~ /^to(?:dsn)?\s$userre/i;
     # Have found the right line
     splice(@{$message->{metadata}}, $linenum, 1);
     $linenum--; # Study the same line again
@@ -555,17 +584,10 @@ sub DeleteRecipients {
 }
 
 
-# ZMailer is always checking its queues anyway, so there is no point
-# doing anything here.
 sub KickMessage {
-  #my(@ids) = @_;
-  #my($idlist);
-  #
-  #$idlist = join(' -qI', @ids);
-  #$idlist .= ' &' if MailScanner::Config::Value('deliverinbackground');
-  ###print STDERR "About to do \"Sendmail2 -qI$idlist\"\n";
-  #system(MailScanner::Config::Value('sendmail2') . ' -qI' . $idlist);
-  ;
+  # ZMailer is always checking its queues anyway, so there is no point
+  # doing anything here.
+  return 1;
 }
 
 #REVISO LEOH
@@ -576,6 +598,22 @@ sub CreateQf {
 }
 
 
+# Append, add or replace a given header with a given value.
+sub AddMultipleHeaderName {
+  my $this = shift;
+  my($message, $headername, $headervalue, $separator) = @_;
+
+  my($multiple) = MailScanner::Config::Value('multipleheaders', $message);
+  $this->AppendHeader ($message, $headername, $headervalue, $separator)
+    if $multiple eq 'append';
+
+  $this->AddHeader    ($message, $headername, $headervalue)
+    if $multiple eq 'add';
+
+  $this->ReplaceHeader($message, $headername, $headervalue)
+    if $multiple eq 'replace';
+}
+
 # Append, add or replace a given header with a given value.
 sub AddMultipleHeader {
   my $this = shift;
@@ -706,7 +744,11 @@ sub CreateBatch {
     $invalidfiles = "";
 
     # Loop through each of the inq directories
-    foreach $queuedirname (@queuedirnames) {
+    # Patch to combat starving in emergency queue mode
+    # foreach $queuedirname (@queuedirnames) {
+    my @aux_queuedirnames=@queuedirnames;
+    while( defined($queuedirname=splice(@aux_queuedirnames,
+          ($UnsortedBatchesLeft<=0 ? 0 :int(rand(@aux_queuedirnames))),1))) {
       #print STDERR "Scanning dir $queuedirname\n";
       unless (chdir $queuedirname) {
         MailScanner::Log::WarnLog("Cannot cd to dir %s to read messages, %s",
@@ -787,7 +829,7 @@ sub CreateBatch {
       $batch->{messages}{"$id"} = $newmessage;
       $batchempty = 0;
 
-      if (MailScanner::Config::Value("virusscan", $newmessage)) {
+      if (MailScanner::Config::Value("virusscan", $newmessage) =~ /1/) {
         $newmessage->NeedsScanning(1);
         $DirtyMsgs++;
         $DirtyBytes += $newmessage->{size};
